#include "ContactsBinding.h"
#include "ExternalBinding.h"
#include "ResourceStatsBinding.h"
#include "UnionTypes.h"
#include "mozilla/dom/AudioTrack.h"
#include "mozilla/dom/CanvasGradient.h"
#include "mozilla/dom/CanvasPattern.h"
#include "mozilla/dom/Directory.h"
#include "mozilla/dom/Element.h"
#include "mozilla/dom/Event.h"
#include "mozilla/dom/File.h"
#include "mozilla/dom/HTMLCanvasElement.h"
#include "mozilla/dom/HTMLImageElement.h"
#include "mozilla/dom/HTMLOptGroupElement.h"
#include "mozilla/dom/HTMLOptionElement.h"
#include "mozilla/dom/HTMLVideoElement.h"
#include "mozilla/dom/Headers.h"
#include "mozilla/dom/MessagePort.h"
#include "mozilla/dom/PrimitiveConversions.h"
#include "mozilla/dom/RadioNodeList.h"
#include "mozilla/dom/Request.h"
#include "mozilla/dom/TelephonyCall.h"
#include "mozilla/dom/TelephonyCallGroup.h"
#include "mozilla/dom/TextTrack.h"
#include "mozilla/dom/URLSearchParams.h"
#include "mozilla/dom/VideoTrack.h"
#include "mozilla/dom/indexedDB/IDBCursor.h"
#include "mozilla/dom/indexedDB/IDBIndex.h"
#include "mozilla/dom/indexedDB/IDBObjectStore.h"
#include "nsContentList.h"
#include "nsFormData.h"
#include "nsGenericHTMLElement.h"
#include "nsIDOMMozMmsMessage.h"
#include "nsIDOMMozSmsMessage.h"
#include "nsIDOMWindow.h"
#include "nsIDocument.h"
#include "nsINode.h"
#include "nsTextNode.h"

namespace mozilla {
namespace dom {
namespace indexedDB {
} // namespace indexedDB
} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {
void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningArrayBufferOrArrayBufferViewOrBlobOrString& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsBlob()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsBlob(), "mBlob", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsURLSearchParams()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsURLSearchParams(), "mURLSearchParams", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningArrayBufferViewOrBlobOrStringOrFormData& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsBlob()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsBlob(), "mBlob", aFlags);
  } else if (aUnion.IsFormData()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsFormData(), "mFormData", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningEventOrString& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsEvent()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsEvent(), "mEvent", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningExternalOrWindowProxy& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsExternal()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsExternal(), "mExternal", aFlags);
  } else if (aUnion.IsWindowProxy()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsWindowProxy(), "mWindowProxy", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningHTMLElementOrLong& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsHTMLElement()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsHTMLElement(), "mHTMLElement", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsHTMLImageElement()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsHTMLImageElement(), "mHTMLImageElement", aFlags);
  } else if (aUnion.IsHTMLCanvasElement()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsHTMLCanvasElement(), "mHTMLCanvasElement", aFlags);
  } else if (aUnion.IsHTMLVideoElement()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsHTMLVideoElement(), "mHTMLVideoElement", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningHTMLOptionElementOrHTMLOptGroupElement& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsHTMLOptionElement()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsHTMLOptionElement(), "mHTMLOptionElement", aFlags);
  } else if (aUnion.IsHTMLOptGroupElement()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsHTMLOptGroupElement(), "mHTMLOptGroupElement", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsHeaders()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsHeaders(), "mHeaders", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningIDBObjectStoreOrIDBIndex& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsIDBObjectStore()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsIDBObjectStore(), "mIDBObjectStore", aFlags);
  } else if (aUnion.IsIDBIndex()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsIDBIndex(), "mIDBIndex", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningIDBObjectStoreOrIDBIndexOrIDBCursor& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsIDBObjectStore()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsIDBObjectStore(), "mIDBObjectStore", aFlags);
  } else if (aUnion.IsIDBIndex()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsIDBIndex(), "mIDBIndex", aFlags);
  } else if (aUnion.IsIDBCursor()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsIDBCursor(), "mIDBCursor", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningLongOrMozSmsMessageOrMozMmsMessage& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsMozSmsMessage()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsMozSmsMessage(), "mMozSmsMessage", aFlags);
  } else if (aUnion.IsMozMmsMessage()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsMozMmsMessage(), "mMozMmsMessage", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningMozSmsMessageOrMozMmsMessage& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsMozSmsMessage()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsMozSmsMessage(), "mMozSmsMessage", aFlags);
  } else if (aUnion.IsMozMmsMessage()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsMozMmsMessage(), "mMozMmsMessage", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningNetworkStatsDataOrPowerStatsData& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsNetworkStatsData()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsNetworkStatsData(), "mNetworkStatsData", aFlags);
  } else if (aUnion.IsPowerStatsData()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsPowerStatsData(), "mPowerStatsData", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningNodeOrHTMLCollection& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsNode()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsNode(), "mNode", aFlags);
  } else if (aUnion.IsHTMLCollection()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsHTMLCollection(), "mHTMLCollection", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningRadioNodeListOrElement& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsRadioNodeList()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsRadioNodeList(), "mRadioNodeList", aFlags);
  } else if (aUnion.IsElement()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsElement(), "mElement", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningRequestOrScalarValueString& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsRequest()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsRequest(), "mRequest", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningStringOrBlobOrArrayBufferOrArrayBufferView& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsBlob()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsBlob(), "mBlob", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningStringOrCanvasGradientOrCanvasPattern& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsCanvasGradient()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsCanvasGradient(), "mCanvasGradient", aFlags);
  } else if (aUnion.IsCanvasPattern()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsCanvasPattern(), "mCanvasPattern", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningStringOrFileOrDirectory& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsFile()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsFile(), "mFile", aFlags);
  } else if (aUnion.IsDirectory()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsDirectory(), "mDirectory", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningTelephonyCallOrTelephonyCallGroup& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsTelephonyCall()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsTelephonyCall(), "mTelephonyCall", aFlags);
  } else if (aUnion.IsTelephonyCallGroup()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsTelephonyCallGroup(), "mTelephonyCallGroup", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningTextOrElementOrDocument& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsText()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsText(), "mText", aFlags);
  } else if (aUnion.IsElement()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsElement(), "mElement", aFlags);
  } else if (aUnion.IsDocument()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsDocument(), "mDocument", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningVideoTrackOrAudioTrackOrTextTrack& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsVideoTrack()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsVideoTrack(), "mVideoTrack", aFlags);
  } else if (aUnion.IsAudioTrack()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsAudioTrack(), "mAudioTrack", aFlags);
  } else if (aUnion.IsTextTrack()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsTextTrack(), "mTextTrack", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningWindowProxyOrMessagePort& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsWindowProxy()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsWindowProxy(), "mWindowProxy", aFlags);
  } else if (aUnion.IsMessagePort()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsMessagePort(), "mMessagePort", aFlags);
  }
}

void
ImplCycleCollectionTraverse(nsCycleCollectionTraversalCallback& aCallback, OwningmozContactOrString& aUnion, const char* aName, uint32_t aFlags)
{
  if (aUnion.IsmozContact()) {
    ImplCycleCollectionTraverse(aCallback, aUnion.GetAsmozContact(), "mmozContact", aFlags);
  }
}

void
ImplCycleCollectionUnlink(OwningArrayBufferOrArrayBufferViewOrBlobOrString& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningArrayBufferViewOrBlobOrStringOrFormData& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningEventOrString& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningExternalOrWindowProxy& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningHTMLElementOrLong& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningHTMLOptionElementOrHTMLOptGroupElement& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningIDBObjectStoreOrIDBIndex& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningIDBObjectStoreOrIDBIndexOrIDBCursor& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningLongOrMozSmsMessageOrMozMmsMessage& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningMozSmsMessageOrMozMmsMessage& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningNetworkStatsDataOrPowerStatsData& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningNodeOrHTMLCollection& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningRadioNodeListOrElement& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningRequestOrScalarValueString& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningStringOrBlobOrArrayBufferOrArrayBufferView& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningStringOrCanvasGradientOrCanvasPattern& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningStringOrFileOrDirectory& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningTelephonyCallOrTelephonyCallGroup& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningTextOrElementOrDocument& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningVideoTrackOrAudioTrackOrTextTrack& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningWindowProxyOrMessagePort& aUnion)
{
  aUnion.Uninit();
}

void
ImplCycleCollectionUnlink(OwningmozContactOrString& aUnion)
{
  aUnion.Uninit();
}

bool
ArrayBufferOrArrayBufferViewOrBlobOrString::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eArrayBuffer: {
      rval.setObject(*mValue.mArrayBuffer.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eArrayBufferView: {
      rval.setObject(*mValue.mArrayBufferView.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eBlob: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mBlob.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
ArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eArrayBuffer: {
      rval.setObject(*mValue.mArrayBuffer.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eArrayBufferView: {
      rval.setObject(*mValue.mArrayBufferView.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eScalarValueString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mScalarValueString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eURLSearchParams: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mURLSearchParams.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
ArrayBufferViewOrArrayBuffer::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eArrayBufferView: {
      rval.setObject(*mValue.mArrayBufferView.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eArrayBuffer: {
      rval.setObject(*mValue.mArrayBuffer.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
ArrayBufferViewOrBlobOrStringOrFormData::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eArrayBufferView: {
      rval.setObject(*mValue.mArrayBufferView.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eBlob: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mBlob.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eFormData: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mFormData.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
BooleanOrMediaTrackConstraints::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eBoolean: {
      rval.setBoolean(mValue.mBoolean.Value());
      return true;
      break;
    }
    case eMediaTrackConstraints: {
      if (!mValue.mMediaTrackConstraints.Value().ToObjectInternal(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
EventOrString::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eEvent: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mEvent.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
ExternalOrWindowProxy::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eExternal: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mExternal.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eWindowProxy: {
      if (!WrapObject(cx, mValue.mWindowProxy.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
HTMLElementOrLong::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eHTMLElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eLong: {
      rval.setInt32(int32_t(mValue.mLong.Value()));
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
HTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eHTMLImageElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLImageElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eHTMLCanvasElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLCanvasElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eHTMLVideoElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLVideoElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
HTMLOptionElementOrHTMLOptGroupElement::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eHTMLOptionElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLOptionElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eHTMLOptGroupElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLOptGroupElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
HeadersOrByteStringSequenceSequenceOrByteStringMozMap::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eHeaders: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHeaders.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eByteStringSequenceSequence: {

      uint32_t length = mValue.mByteStringSequenceSequence.Value().Length();
      JS::Rooted<JSObject*> returnArray(cx, JS_NewArrayObject(cx, length));
      if (!returnArray) {
        return false;
      }
      // Scope for 'tmp'
      {
        JS::Rooted<JS::Value> tmp(cx);
        for (uint32_t sequenceIdx0 = 0; sequenceIdx0 < length; ++sequenceIdx0) {
          // Control block to let us common up the JS_DefineElement calls when there
          // are different ways to succeed at wrapping the object.
          do {

            uint32_t length = mValue.mByteStringSequenceSequence.Value()[sequenceIdx0].Length();
            JS::Rooted<JSObject*> returnArray(cx, JS_NewArrayObject(cx, length));
            if (!returnArray) {
              return false;
            }
            // Scope for 'tmp'
            {
              JS::Rooted<JS::Value> tmp(cx);
              for (uint32_t sequenceIdx1 = 0; sequenceIdx1 < length; ++sequenceIdx1) {
                // Control block to let us common up the JS_DefineElement calls when there
                // are different ways to succeed at wrapping the object.
                do {
                  if (!NonVoidByteStringToJsval(cx, mValue.mByteStringSequenceSequence.Value()[sequenceIdx0][sequenceIdx1], &tmp)) {
                    return false;
                  }
                  break;
                } while (0);
                if (!JS_DefineElement(cx, returnArray, sequenceIdx1, tmp,
                                      JSPROP_ENUMERATE)) {
                  return false;
                }
              }
            }
            tmp.setObject(*returnArray);
            break;
          } while (0);
          if (!JS_DefineElement(cx, returnArray, sequenceIdx0, tmp,
                                JSPROP_ENUMERATE)) {
            return false;
          }
        }
      }
      rval.setObject(*returnArray);
      return true;
      break;
    }
    case eByteStringMozMap: {

      nsTArray<nsString> keys;
      mValue.mByteStringMozMap.Value().GetKeys(keys);
      JS::Rooted<JSObject*> returnObj(cx, JS_NewObject(cx, nullptr, JS::NullPtr(), JS::NullPtr()));
      if (!returnObj) {
        return false;
      }
      // Scope for 'tmp'
      {
        JS::Rooted<JS::Value> tmp(cx);
        for (size_t idx = 0; idx < keys.Length(); ++idx) {
          auto& mozMapValue0 = mValue.mByteStringMozMap.Value().Get(keys[idx]);
          // Control block to let us common up the JS_DefineUCProperty calls when there
          // are different ways to succeed at wrapping the value.
          do {
            if (!NonVoidByteStringToJsval(cx, mozMapValue0, &tmp)) {
              return false;
            }
            break;
          } while (0);
          if (!JS_DefineUCProperty(cx, returnObj, keys[idx].get(),
                                   keys[idx].Length(), tmp,
                                   JSPROP_ENUMERATE)) {
            return false;
          }
        }
      }
      rval.setObject(*returnObj);
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
IDBObjectStoreOrIDBIndex::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eIDBObjectStore: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mIDBObjectStore.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eIDBIndex: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mIDBIndex.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
IDBObjectStoreOrIDBIndexOrIDBCursor::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eIDBObjectStore: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mIDBObjectStore.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eIDBIndex: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mIDBIndex.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eIDBCursor: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mIDBCursor.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
LongOrAutoKeyword::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eLong: {
      rval.setInt32(int32_t(mValue.mLong.Value()));
      return true;
      break;
    }
    case eAutoKeyword: {
      {
        // Scope for resultStr
        MOZ_ASSERT(uint32_t(mValue.mAutoKeyword.Value()) < ArrayLength(AutoKeywordValues::strings));
        JSString* resultStr = JS_NewStringCopyN(cx, AutoKeywordValues::strings[uint32_t(mValue.mAutoKeyword.Value())].value, AutoKeywordValues::strings[uint32_t(mValue.mAutoKeyword.Value())].length);
        if (!resultStr) {
          return false;
        }
        rval.setString(resultStr);
        return true;
      }
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
LongOrMozSmsMessageOrMozMmsMessage::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eLong: {
      rval.setInt32(int32_t(mValue.mLong.Value()));
      return true;
      break;
    }
    case eMozSmsMessage: {
      if (!WrapObject(cx, mValue.mMozSmsMessage.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eMozMmsMessage: {
      if (!WrapObject(cx, mValue.mMozMmsMessage.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
MozSmsMessageOrMozMmsMessage::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eMozSmsMessage: {
      if (!WrapObject(cx, mValue.mMozSmsMessage.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eMozMmsMessage: {
      if (!WrapObject(cx, mValue.mMozMmsMessage.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
NetworkStatsDataOrPowerStatsData::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eNetworkStatsData: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mNetworkStatsData.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case ePowerStatsData: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mPowerStatsData.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
NodeOrHTMLCollection::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eNode: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mNode.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eHTMLCollection: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLCollection.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
ObjectOrString::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eObject: {
      JS::ExposeObjectToActiveJS(mValue.mObject.Value());
      rval.setObject(*mValue.mObject.Value());
      if (!MaybeWrapObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
RadioNodeListOrElement::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eRadioNodeList: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mRadioNodeList.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
RequestOrScalarValueString::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eRequest: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mRequest.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eScalarValueString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mScalarValueString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
StringOrArrayBufferOrArrayBufferView::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eArrayBuffer: {
      rval.setObject(*mValue.mArrayBuffer.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eArrayBufferView: {
      rval.setObject(*mValue.mArrayBufferView.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
StringOrBlobOrArrayBufferOrArrayBufferView::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eBlob: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mBlob.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eArrayBuffer: {
      rval.setObject(*mValue.mArrayBuffer.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eArrayBufferView: {
      rval.setObject(*mValue.mArrayBufferView.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
StringOrCanvasGradientOrCanvasPattern::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eCanvasGradient: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mCanvasGradient.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eCanvasPattern: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mCanvasPattern.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
StringOrFileOrDirectory::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eFile: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mFile.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eDirectory: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mDirectory.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
StringOrInstallTriggerData::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eInstallTriggerData: {
      if (!mValue.mInstallTriggerData.Value().ToObjectInternal(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
StringOrUnsignedLong::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eUnsignedLong: {
      rval.setNumber(mValue.mUnsignedLong.Value());
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
TelephonyCallOrTelephonyCallGroup::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eTelephonyCall: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mTelephonyCall.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eTelephonyCallGroup: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mTelephonyCallGroup.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
TextOrElementOrDocument::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eText: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mText.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eDocument: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mDocument.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
UnsignedLongOrString::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eUnsignedLong: {
      rval.setNumber(mValue.mUnsignedLong.Value());
      return true;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
UnsignedShortOrObject::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eUnsignedShort: {
      rval.setInt32(int32_t(mValue.mUnsignedShort.Value()));
      return true;
      break;
    }
    case eObject: {
      JS::ExposeObjectToActiveJS(mValue.mObject.Value());
      rval.setObject(*mValue.mObject.Value());
      if (!MaybeWrapObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
VideoTrackOrAudioTrackOrTextTrack::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eVideoTrack: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mVideoTrack.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eAudioTrack: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mAudioTrack.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eTextTrack: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mTextTrack.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
WindowProxyOrMessagePort::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eWindowProxy: {
      if (!WrapObject(cx, mValue.mWindowProxy.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eMessagePort: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mMessagePort.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

bool
mozContactOrString::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case emozContact: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mmozContact.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

ArrayBuffer&
OwningArrayBufferOrArrayBufferViewOrBlobOrString::RawSetAsArrayBuffer()
{
  if (mType == eArrayBuffer) {
    return mValue.mArrayBuffer.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eArrayBuffer;
  return mValue.mArrayBuffer.SetValue();
}

ArrayBuffer&
OwningArrayBufferOrArrayBufferViewOrBlobOrString::SetAsArrayBuffer()
{
  if (mType == eArrayBuffer) {
    return mValue.mArrayBuffer.Value();
  }
  Uninit();
  mType = eArrayBuffer;
  return mValue.mArrayBuffer.SetValue();
}

bool
OwningArrayBufferOrArrayBufferViewOrBlobOrString::TrySetToArrayBuffer(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    ArrayBuffer& memberSlot = RawSetAsArrayBuffer();
    if (!memberSlot.Init(&value.toObject())) {
      DestroyArrayBuffer();
      tryNext = true;
      return true;
    }
  }
  return true;
}

void
OwningArrayBufferOrArrayBufferViewOrBlobOrString::DestroyArrayBuffer()
{
  MOZ_ASSERT(IsArrayBuffer(), "Wrong type!");
  mValue.mArrayBuffer.Destroy();
  mType = eUninitialized;
}




ArrayBufferView&
OwningArrayBufferOrArrayBufferViewOrBlobOrString::RawSetAsArrayBufferView()
{
  if (mType == eArrayBufferView) {
    return mValue.mArrayBufferView.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eArrayBufferView;
  return mValue.mArrayBufferView.SetValue();
}

ArrayBufferView&
OwningArrayBufferOrArrayBufferViewOrBlobOrString::SetAsArrayBufferView()
{
  if (mType == eArrayBufferView) {
    return mValue.mArrayBufferView.Value();
  }
  Uninit();
  mType = eArrayBufferView;
  return mValue.mArrayBufferView.SetValue();
}

bool
OwningArrayBufferOrArrayBufferViewOrBlobOrString::TrySetToArrayBufferView(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    ArrayBufferView& memberSlot = RawSetAsArrayBufferView();
    if (!memberSlot.Init(&value.toObject())) {
      DestroyArrayBufferView();
      tryNext = true;
      return true;
    }
  }
  return true;
}

void
OwningArrayBufferOrArrayBufferViewOrBlobOrString::DestroyArrayBufferView()
{
  MOZ_ASSERT(IsArrayBufferView(), "Wrong type!");
  mValue.mArrayBufferView.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::File>&
OwningArrayBufferOrArrayBufferViewOrBlobOrString::RawSetAsBlob()
{
  if (mType == eBlob) {
    return mValue.mBlob.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eBlob;
  return mValue.mBlob.SetValue();
}

OwningNonNull<mozilla::dom::File>&
OwningArrayBufferOrArrayBufferViewOrBlobOrString::SetAsBlob()
{
  if (mType == eBlob) {
    return mValue.mBlob.Value();
  }
  Uninit();
  mType = eBlob;
  return mValue.mBlob.SetValue();
}

bool
OwningArrayBufferOrArrayBufferViewOrBlobOrString::TrySetToBlob(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::File>& memberSlot = RawSetAsBlob();
    {
      nsresult rv = UnwrapObject<prototypes::id::Blob, mozilla::dom::File>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyBlob();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningArrayBufferOrArrayBufferViewOrBlobOrString::DestroyBlob()
{
  MOZ_ASSERT(IsBlob(), "Wrong type!");
  mValue.mBlob.Destroy();
  mType = eUninitialized;
}




nsString&
OwningArrayBufferOrArrayBufferViewOrBlobOrString::RawSetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eString;
  return mValue.mString.SetValue();
}

nsString&
OwningArrayBufferOrArrayBufferViewOrBlobOrString::SetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  Uninit();
  mType = eString;
  return mValue.mString.SetValue();
}

bool
OwningArrayBufferOrArrayBufferViewOrBlobOrString::TrySetToString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
  }
  return true;
}


void
OwningArrayBufferOrArrayBufferViewOrBlobOrString::DestroyString()
{
  MOZ_ASSERT(IsString(), "Wrong type!");
  mValue.mString.Destroy();
  mType = eUninitialized;
}




void
OwningArrayBufferOrArrayBufferViewOrBlobOrString::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eArrayBuffer: {
      DestroyArrayBuffer();
      break;
    }
    case eArrayBufferView: {
      DestroyArrayBufferView();
      break;
    }
    case eBlob: {
      DestroyBlob();
      break;
    }
    case eString: {
      DestroyString();
      break;
    }
  }
}

bool
OwningArrayBufferOrArrayBufferViewOrBlobOrString::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eArrayBuffer: {
      rval.setObject(*mValue.mArrayBuffer.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eArrayBufferView: {
      rval.setObject(*mValue.mArrayBufferView.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eBlob: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mBlob.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningArrayBufferOrArrayBufferViewOrBlobOrString::TraceUnion(JSTracer* trc)
{
  switch (mType) {
    case eArrayBuffer: {
      mValue.mArrayBuffer.Value().TraceSelf(trc);
      break;
    }
    case eArrayBufferView: {
      mValue.mArrayBufferView.Value().TraceSelf(trc);
      break;
    }
    default: {
      break;
    }
  }
}

ArrayBuffer&
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::RawSetAsArrayBuffer()
{
  if (mType == eArrayBuffer) {
    return mValue.mArrayBuffer.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eArrayBuffer;
  return mValue.mArrayBuffer.SetValue();
}

ArrayBuffer&
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::SetAsArrayBuffer()
{
  if (mType == eArrayBuffer) {
    return mValue.mArrayBuffer.Value();
  }
  Uninit();
  mType = eArrayBuffer;
  return mValue.mArrayBuffer.SetValue();
}

bool
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::TrySetToArrayBuffer(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    ArrayBuffer& memberSlot = RawSetAsArrayBuffer();
    if (!memberSlot.Init(&value.toObject())) {
      DestroyArrayBuffer();
      tryNext = true;
      return true;
    }
  }
  return true;
}

void
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::DestroyArrayBuffer()
{
  MOZ_ASSERT(IsArrayBuffer(), "Wrong type!");
  mValue.mArrayBuffer.Destroy();
  mType = eUninitialized;
}




ArrayBufferView&
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::RawSetAsArrayBufferView()
{
  if (mType == eArrayBufferView) {
    return mValue.mArrayBufferView.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eArrayBufferView;
  return mValue.mArrayBufferView.SetValue();
}

ArrayBufferView&
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::SetAsArrayBufferView()
{
  if (mType == eArrayBufferView) {
    return mValue.mArrayBufferView.Value();
  }
  Uninit();
  mType = eArrayBufferView;
  return mValue.mArrayBufferView.SetValue();
}

bool
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::TrySetToArrayBufferView(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    ArrayBufferView& memberSlot = RawSetAsArrayBufferView();
    if (!memberSlot.Init(&value.toObject())) {
      DestroyArrayBufferView();
      tryNext = true;
      return true;
    }
  }
  return true;
}

void
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::DestroyArrayBufferView()
{
  MOZ_ASSERT(IsArrayBufferView(), "Wrong type!");
  mValue.mArrayBufferView.Destroy();
  mType = eUninitialized;
}




nsString&
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::RawSetAsScalarValueString()
{
  if (mType == eScalarValueString) {
    return mValue.mScalarValueString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eScalarValueString;
  return mValue.mScalarValueString.SetValue();
}

nsString&
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::SetAsScalarValueString()
{
  if (mType == eScalarValueString) {
    return mValue.mScalarValueString.Value();
  }
  Uninit();
  mType = eScalarValueString;
  return mValue.mScalarValueString.SetValue();
}

bool
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::TrySetToScalarValueString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsScalarValueString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
    NormalizeScalarValueString(cx, memberSlot);
  }
  return true;
}


void
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::DestroyScalarValueString()
{
  MOZ_ASSERT(IsScalarValueString(), "Wrong type!");
  mValue.mScalarValueString.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::URLSearchParams>&
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::RawSetAsURLSearchParams()
{
  if (mType == eURLSearchParams) {
    return mValue.mURLSearchParams.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eURLSearchParams;
  return mValue.mURLSearchParams.SetValue();
}

OwningNonNull<mozilla::dom::URLSearchParams>&
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::SetAsURLSearchParams()
{
  if (mType == eURLSearchParams) {
    return mValue.mURLSearchParams.Value();
  }
  Uninit();
  mType = eURLSearchParams;
  return mValue.mURLSearchParams.SetValue();
}

bool
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::TrySetToURLSearchParams(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::URLSearchParams>& memberSlot = RawSetAsURLSearchParams();
    {
      nsresult rv = UnwrapObject<prototypes::id::URLSearchParams, mozilla::dom::URLSearchParams>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyURLSearchParams();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::DestroyURLSearchParams()
{
  MOZ_ASSERT(IsURLSearchParams(), "Wrong type!");
  mValue.mURLSearchParams.Destroy();
  mType = eUninitialized;
}




void
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eArrayBuffer: {
      DestroyArrayBuffer();
      break;
    }
    case eArrayBufferView: {
      DestroyArrayBufferView();
      break;
    }
    case eScalarValueString: {
      DestroyScalarValueString();
      break;
    }
    case eURLSearchParams: {
      DestroyURLSearchParams();
      break;
    }
  }
}

bool
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eArrayBuffer: {
      rval.setObject(*mValue.mArrayBuffer.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eArrayBufferView: {
      rval.setObject(*mValue.mArrayBufferView.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eScalarValueString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mScalarValueString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eURLSearchParams: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mURLSearchParams.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningArrayBufferOrArrayBufferViewOrScalarValueStringOrURLSearchParams::TraceUnion(JSTracer* trc)
{
  switch (mType) {
    case eArrayBuffer: {
      mValue.mArrayBuffer.Value().TraceSelf(trc);
      break;
    }
    case eArrayBufferView: {
      mValue.mArrayBufferView.Value().TraceSelf(trc);
      break;
    }
    default: {
      break;
    }
  }
}

ArrayBufferView&
OwningArrayBufferViewOrArrayBuffer::RawSetAsArrayBufferView()
{
  if (mType == eArrayBufferView) {
    return mValue.mArrayBufferView.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eArrayBufferView;
  return mValue.mArrayBufferView.SetValue();
}

ArrayBufferView&
OwningArrayBufferViewOrArrayBuffer::SetAsArrayBufferView()
{
  if (mType == eArrayBufferView) {
    return mValue.mArrayBufferView.Value();
  }
  Uninit();
  mType = eArrayBufferView;
  return mValue.mArrayBufferView.SetValue();
}

bool
OwningArrayBufferViewOrArrayBuffer::TrySetToArrayBufferView(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    ArrayBufferView& memberSlot = RawSetAsArrayBufferView();
    if (!memberSlot.Init(&value.toObject())) {
      DestroyArrayBufferView();
      tryNext = true;
      return true;
    }
  }
  return true;
}

void
OwningArrayBufferViewOrArrayBuffer::DestroyArrayBufferView()
{
  MOZ_ASSERT(IsArrayBufferView(), "Wrong type!");
  mValue.mArrayBufferView.Destroy();
  mType = eUninitialized;
}




ArrayBuffer&
OwningArrayBufferViewOrArrayBuffer::RawSetAsArrayBuffer()
{
  if (mType == eArrayBuffer) {
    return mValue.mArrayBuffer.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eArrayBuffer;
  return mValue.mArrayBuffer.SetValue();
}

ArrayBuffer&
OwningArrayBufferViewOrArrayBuffer::SetAsArrayBuffer()
{
  if (mType == eArrayBuffer) {
    return mValue.mArrayBuffer.Value();
  }
  Uninit();
  mType = eArrayBuffer;
  return mValue.mArrayBuffer.SetValue();
}

bool
OwningArrayBufferViewOrArrayBuffer::TrySetToArrayBuffer(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    ArrayBuffer& memberSlot = RawSetAsArrayBuffer();
    if (!memberSlot.Init(&value.toObject())) {
      DestroyArrayBuffer();
      tryNext = true;
      return true;
    }
  }
  return true;
}

void
OwningArrayBufferViewOrArrayBuffer::DestroyArrayBuffer()
{
  MOZ_ASSERT(IsArrayBuffer(), "Wrong type!");
  mValue.mArrayBuffer.Destroy();
  mType = eUninitialized;
}




void
OwningArrayBufferViewOrArrayBuffer::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eArrayBufferView: {
      DestroyArrayBufferView();
      break;
    }
    case eArrayBuffer: {
      DestroyArrayBuffer();
      break;
    }
  }
}

bool
OwningArrayBufferViewOrArrayBuffer::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eArrayBufferView: {
      rval.setObject(*mValue.mArrayBufferView.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eArrayBuffer: {
      rval.setObject(*mValue.mArrayBuffer.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningArrayBufferViewOrArrayBuffer::TraceUnion(JSTracer* trc)
{
  switch (mType) {
    case eArrayBufferView: {
      mValue.mArrayBufferView.Value().TraceSelf(trc);
      break;
    }
    case eArrayBuffer: {
      mValue.mArrayBuffer.Value().TraceSelf(trc);
      break;
    }
    default: {
      break;
    }
  }
}

ArrayBufferView&
OwningArrayBufferViewOrBlobOrStringOrFormData::RawSetAsArrayBufferView()
{
  if (mType == eArrayBufferView) {
    return mValue.mArrayBufferView.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eArrayBufferView;
  return mValue.mArrayBufferView.SetValue();
}

ArrayBufferView&
OwningArrayBufferViewOrBlobOrStringOrFormData::SetAsArrayBufferView()
{
  if (mType == eArrayBufferView) {
    return mValue.mArrayBufferView.Value();
  }
  Uninit();
  mType = eArrayBufferView;
  return mValue.mArrayBufferView.SetValue();
}

bool
OwningArrayBufferViewOrBlobOrStringOrFormData::TrySetToArrayBufferView(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    ArrayBufferView& memberSlot = RawSetAsArrayBufferView();
    if (!memberSlot.Init(&value.toObject())) {
      DestroyArrayBufferView();
      tryNext = true;
      return true;
    }
  }
  return true;
}

void
OwningArrayBufferViewOrBlobOrStringOrFormData::DestroyArrayBufferView()
{
  MOZ_ASSERT(IsArrayBufferView(), "Wrong type!");
  mValue.mArrayBufferView.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::File>&
OwningArrayBufferViewOrBlobOrStringOrFormData::RawSetAsBlob()
{
  if (mType == eBlob) {
    return mValue.mBlob.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eBlob;
  return mValue.mBlob.SetValue();
}

OwningNonNull<mozilla::dom::File>&
OwningArrayBufferViewOrBlobOrStringOrFormData::SetAsBlob()
{
  if (mType == eBlob) {
    return mValue.mBlob.Value();
  }
  Uninit();
  mType = eBlob;
  return mValue.mBlob.SetValue();
}

bool
OwningArrayBufferViewOrBlobOrStringOrFormData::TrySetToBlob(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::File>& memberSlot = RawSetAsBlob();
    {
      nsresult rv = UnwrapObject<prototypes::id::Blob, mozilla::dom::File>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyBlob();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningArrayBufferViewOrBlobOrStringOrFormData::DestroyBlob()
{
  MOZ_ASSERT(IsBlob(), "Wrong type!");
  mValue.mBlob.Destroy();
  mType = eUninitialized;
}




nsString&
OwningArrayBufferViewOrBlobOrStringOrFormData::RawSetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eString;
  return mValue.mString.SetValue();
}

nsString&
OwningArrayBufferViewOrBlobOrStringOrFormData::SetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  Uninit();
  mType = eString;
  return mValue.mString.SetValue();
}

bool
OwningArrayBufferViewOrBlobOrStringOrFormData::TrySetToString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
  }
  return true;
}


void
OwningArrayBufferViewOrBlobOrStringOrFormData::DestroyString()
{
  MOZ_ASSERT(IsString(), "Wrong type!");
  mValue.mString.Destroy();
  mType = eUninitialized;
}




OwningNonNull<nsFormData>&
OwningArrayBufferViewOrBlobOrStringOrFormData::RawSetAsFormData()
{
  if (mType == eFormData) {
    return mValue.mFormData.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eFormData;
  return mValue.mFormData.SetValue();
}

OwningNonNull<nsFormData>&
OwningArrayBufferViewOrBlobOrStringOrFormData::SetAsFormData()
{
  if (mType == eFormData) {
    return mValue.mFormData.Value();
  }
  Uninit();
  mType = eFormData;
  return mValue.mFormData.SetValue();
}

bool
OwningArrayBufferViewOrBlobOrStringOrFormData::TrySetToFormData(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<nsFormData>& memberSlot = RawSetAsFormData();
    {
      nsresult rv = UnwrapObject<prototypes::id::FormData, nsFormData>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyFormData();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningArrayBufferViewOrBlobOrStringOrFormData::DestroyFormData()
{
  MOZ_ASSERT(IsFormData(), "Wrong type!");
  mValue.mFormData.Destroy();
  mType = eUninitialized;
}




void
OwningArrayBufferViewOrBlobOrStringOrFormData::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eArrayBufferView: {
      DestroyArrayBufferView();
      break;
    }
    case eBlob: {
      DestroyBlob();
      break;
    }
    case eString: {
      DestroyString();
      break;
    }
    case eFormData: {
      DestroyFormData();
      break;
    }
  }
}

bool
OwningArrayBufferViewOrBlobOrStringOrFormData::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eArrayBufferView: {
      rval.setObject(*mValue.mArrayBufferView.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eBlob: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mBlob.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eFormData: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mFormData.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningArrayBufferViewOrBlobOrStringOrFormData::TraceUnion(JSTracer* trc)
{
  switch (mType) {
    case eArrayBufferView: {
      mValue.mArrayBufferView.Value().TraceSelf(trc);
      break;
    }
    default: {
      break;
    }
  }
}

bool&
OwningBooleanOrMediaTrackConstraints::RawSetAsBoolean()
{
  if (mType == eBoolean) {
    return mValue.mBoolean.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eBoolean;
  return mValue.mBoolean.SetValue();
}

bool&
OwningBooleanOrMediaTrackConstraints::SetAsBoolean()
{
  if (mType == eBoolean) {
    return mValue.mBoolean.Value();
  }
  Uninit();
  mType = eBoolean;
  return mValue.mBoolean.SetValue();
}

bool
OwningBooleanOrMediaTrackConstraints::TrySetToBoolean(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    bool& memberSlot = RawSetAsBoolean();
    if (!ValueToPrimitive<bool, eDefault>(cx, value, &memberSlot)) {
      return false;
    }
  }
  return true;
}

void
OwningBooleanOrMediaTrackConstraints::DestroyBoolean()
{
  MOZ_ASSERT(IsBoolean(), "Wrong type!");
  mValue.mBoolean.Destroy();
  mType = eUninitialized;
}




MediaTrackConstraints&
OwningBooleanOrMediaTrackConstraints::RawSetAsMediaTrackConstraints()
{
  if (mType == eMediaTrackConstraints) {
    return mValue.mMediaTrackConstraints.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eMediaTrackConstraints;
  return mValue.mMediaTrackConstraints.SetValue();
}

MediaTrackConstraints&
OwningBooleanOrMediaTrackConstraints::SetAsMediaTrackConstraints()
{
  if (mType == eMediaTrackConstraints) {
    return mValue.mMediaTrackConstraints.Value();
  }
  Uninit();
  mType = eMediaTrackConstraints;
  return mValue.mMediaTrackConstraints.SetValue();
}

bool
OwningBooleanOrMediaTrackConstraints::TrySetToMediaTrackConstraints(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    MediaTrackConstraints& memberSlot = RawSetAsMediaTrackConstraints();
    if (!IsConvertibleToDictionary(cx, value)) {
      DestroyMediaTrackConstraints();
      tryNext = true;
      return true;
    }

    if (!memberSlot.Init(cx, value, "Member of BooleanOrMediaTrackConstraints", passedToJSImpl)) {
      return false;
    }
  }
  return true;
}

void
OwningBooleanOrMediaTrackConstraints::DestroyMediaTrackConstraints()
{
  MOZ_ASSERT(IsMediaTrackConstraints(), "Wrong type!");
  mValue.mMediaTrackConstraints.Destroy();
  mType = eUninitialized;
}




void
OwningBooleanOrMediaTrackConstraints::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eBoolean: {
      DestroyBoolean();
      break;
    }
    case eMediaTrackConstraints: {
      DestroyMediaTrackConstraints();
      break;
    }
  }
}

bool
OwningBooleanOrMediaTrackConstraints::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eBoolean: {
      rval.setBoolean(mValue.mBoolean.Value());
      return true;
      break;
    }
    case eMediaTrackConstraints: {
      if (!mValue.mMediaTrackConstraints.Value().ToObjectInternal(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningBooleanOrMediaTrackConstraints::TraceUnion(JSTracer* trc)
{
}

void
OwningBooleanOrMediaTrackConstraints::operator=(const OwningBooleanOrMediaTrackConstraints& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eBoolean: {
      SetAsBoolean() = aOther.GetAsBoolean();
      break;
    }
    case eMediaTrackConstraints: {
      SetAsMediaTrackConstraints() = aOther.GetAsMediaTrackConstraints();
      break;
    }
  }
}

OwningNonNull<mozilla::dom::Event>&
OwningEventOrString::RawSetAsEvent()
{
  if (mType == eEvent) {
    return mValue.mEvent.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eEvent;
  return mValue.mEvent.SetValue();
}

OwningNonNull<mozilla::dom::Event>&
OwningEventOrString::SetAsEvent()
{
  if (mType == eEvent) {
    return mValue.mEvent.Value();
  }
  Uninit();
  mType = eEvent;
  return mValue.mEvent.SetValue();
}

bool
OwningEventOrString::TrySetToEvent(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::Event>& memberSlot = RawSetAsEvent();
    {
      nsresult rv = UnwrapObject<prototypes::id::Event, mozilla::dom::Event>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyEvent();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningEventOrString::DestroyEvent()
{
  MOZ_ASSERT(IsEvent(), "Wrong type!");
  mValue.mEvent.Destroy();
  mType = eUninitialized;
}




nsString&
OwningEventOrString::RawSetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eString;
  return mValue.mString.SetValue();
}

nsString&
OwningEventOrString::SetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  Uninit();
  mType = eString;
  return mValue.mString.SetValue();
}

bool
OwningEventOrString::TrySetToString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
  }
  return true;
}


void
OwningEventOrString::DestroyString()
{
  MOZ_ASSERT(IsString(), "Wrong type!");
  mValue.mString.Destroy();
  mType = eUninitialized;
}




void
OwningEventOrString::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eEvent: {
      DestroyEvent();
      break;
    }
    case eString: {
      DestroyString();
      break;
    }
  }
}

bool
OwningEventOrString::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eEvent: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mEvent.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningEventOrString::TraceUnion(JSTracer* trc)
{
}

void
OwningEventOrString::operator=(const OwningEventOrString& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eEvent: {
      SetAsEvent() = aOther.GetAsEvent();
      break;
    }
    case eString: {
      SetAsString() = aOther.GetAsString();
      break;
    }
  }
}

OwningNonNull<mozilla::dom::External>&
OwningExternalOrWindowProxy::RawSetAsExternal()
{
  if (mType == eExternal) {
    return mValue.mExternal.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eExternal;
  return mValue.mExternal.SetValue();
}

OwningNonNull<mozilla::dom::External>&
OwningExternalOrWindowProxy::SetAsExternal()
{
  if (mType == eExternal) {
    return mValue.mExternal.Value();
  }
  Uninit();
  mType = eExternal;
  return mValue.mExternal.SetValue();
}

bool
OwningExternalOrWindowProxy::TrySetToExternal(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::External>& memberSlot = RawSetAsExternal();
    {
      nsresult rv = UnwrapObject<prototypes::id::External, mozilla::dom::External>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyExternal();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningExternalOrWindowProxy::DestroyExternal()
{
  MOZ_ASSERT(IsExternal(), "Wrong type!");
  mValue.mExternal.Destroy();
  mType = eUninitialized;
}




nsRefPtr<nsIDOMWindow>&
OwningExternalOrWindowProxy::RawSetAsWindowProxy()
{
  if (mType == eWindowProxy) {
    return mValue.mWindowProxy.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eWindowProxy;
  return mValue.mWindowProxy.SetValue();
}

nsRefPtr<nsIDOMWindow>&
OwningExternalOrWindowProxy::SetAsWindowProxy()
{
  if (mType == eWindowProxy) {
    return mValue.mWindowProxy.Value();
  }
  Uninit();
  mType = eWindowProxy;
  return mValue.mWindowProxy.SetValue();
}

bool
OwningExternalOrWindowProxy::TrySetToWindowProxy(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsRefPtr<nsIDOMWindow>& memberSlot = RawSetAsWindowProxy();
    nsRefPtr<nsIDOMWindow> mWindowProxyHolder;
    JS::Rooted<JS::Value> tmpVal(cx, value);
    nsIDOMWindow* tmp;
    if (NS_FAILED(UnwrapArg<nsIDOMWindow>(cx, value, &tmp, static_cast<nsIDOMWindow**>(getter_AddRefs(mWindowProxyHolder)), &tmpVal))) {
      DestroyWindowProxy();
      tryNext = true;
      return true;
    }
    MOZ_ASSERT(tmp);
    memberSlot = tmp;
  }
  return true;
}

void
OwningExternalOrWindowProxy::DestroyWindowProxy()
{
  MOZ_ASSERT(IsWindowProxy(), "Wrong type!");
  mValue.mWindowProxy.Destroy();
  mType = eUninitialized;
}




void
OwningExternalOrWindowProxy::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eExternal: {
      DestroyExternal();
      break;
    }
    case eWindowProxy: {
      DestroyWindowProxy();
      break;
    }
  }
}

bool
OwningExternalOrWindowProxy::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eExternal: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mExternal.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eWindowProxy: {
      if (!WrapObject(cx, mValue.mWindowProxy.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningExternalOrWindowProxy::TraceUnion(JSTracer* trc)
{
}

void
OwningExternalOrWindowProxy::operator=(const OwningExternalOrWindowProxy& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eExternal: {
      SetAsExternal() = aOther.GetAsExternal();
      break;
    }
    case eWindowProxy: {
      SetAsWindowProxy() = aOther.GetAsWindowProxy();
      break;
    }
  }
}

OwningNonNull<nsGenericHTMLElement>&
OwningHTMLElementOrLong::RawSetAsHTMLElement()
{
  if (mType == eHTMLElement) {
    return mValue.mHTMLElement.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eHTMLElement;
  return mValue.mHTMLElement.SetValue();
}

OwningNonNull<nsGenericHTMLElement>&
OwningHTMLElementOrLong::SetAsHTMLElement()
{
  if (mType == eHTMLElement) {
    return mValue.mHTMLElement.Value();
  }
  Uninit();
  mType = eHTMLElement;
  return mValue.mHTMLElement.SetValue();
}

bool
OwningHTMLElementOrLong::TrySetToHTMLElement(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<nsGenericHTMLElement>& memberSlot = RawSetAsHTMLElement();
    {
      nsresult rv = UnwrapObject<prototypes::id::HTMLElement, nsGenericHTMLElement>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyHTMLElement();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningHTMLElementOrLong::DestroyHTMLElement()
{
  MOZ_ASSERT(IsHTMLElement(), "Wrong type!");
  mValue.mHTMLElement.Destroy();
  mType = eUninitialized;
}




int32_t&
OwningHTMLElementOrLong::RawSetAsLong()
{
  if (mType == eLong) {
    return mValue.mLong.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eLong;
  return mValue.mLong.SetValue();
}

int32_t&
OwningHTMLElementOrLong::SetAsLong()
{
  if (mType == eLong) {
    return mValue.mLong.Value();
  }
  Uninit();
  mType = eLong;
  return mValue.mLong.SetValue();
}

bool
OwningHTMLElementOrLong::TrySetToLong(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    int32_t& memberSlot = RawSetAsLong();
    if (!ValueToPrimitive<int32_t, eDefault>(cx, value, &memberSlot)) {
      return false;
    }
  }
  return true;
}

void
OwningHTMLElementOrLong::DestroyLong()
{
  MOZ_ASSERT(IsLong(), "Wrong type!");
  mValue.mLong.Destroy();
  mType = eUninitialized;
}




void
OwningHTMLElementOrLong::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eHTMLElement: {
      DestroyHTMLElement();
      break;
    }
    case eLong: {
      DestroyLong();
      break;
    }
  }
}

bool
OwningHTMLElementOrLong::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eHTMLElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eLong: {
      rval.setInt32(int32_t(mValue.mLong.Value()));
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningHTMLElementOrLong::TraceUnion(JSTracer* trc)
{
}

void
OwningHTMLElementOrLong::operator=(const OwningHTMLElementOrLong& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eHTMLElement: {
      SetAsHTMLElement() = aOther.GetAsHTMLElement();
      break;
    }
    case eLong: {
      SetAsLong() = aOther.GetAsLong();
      break;
    }
  }
}

OwningNonNull<mozilla::dom::HTMLImageElement>&
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::RawSetAsHTMLImageElement()
{
  if (mType == eHTMLImageElement) {
    return mValue.mHTMLImageElement.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eHTMLImageElement;
  return mValue.mHTMLImageElement.SetValue();
}

OwningNonNull<mozilla::dom::HTMLImageElement>&
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::SetAsHTMLImageElement()
{
  if (mType == eHTMLImageElement) {
    return mValue.mHTMLImageElement.Value();
  }
  Uninit();
  mType = eHTMLImageElement;
  return mValue.mHTMLImageElement.SetValue();
}

bool
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::TrySetToHTMLImageElement(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::HTMLImageElement>& memberSlot = RawSetAsHTMLImageElement();
    {
      nsresult rv = UnwrapObject<prototypes::id::HTMLImageElement, mozilla::dom::HTMLImageElement>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyHTMLImageElement();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::DestroyHTMLImageElement()
{
  MOZ_ASSERT(IsHTMLImageElement(), "Wrong type!");
  mValue.mHTMLImageElement.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::HTMLCanvasElement>&
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::RawSetAsHTMLCanvasElement()
{
  if (mType == eHTMLCanvasElement) {
    return mValue.mHTMLCanvasElement.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eHTMLCanvasElement;
  return mValue.mHTMLCanvasElement.SetValue();
}

OwningNonNull<mozilla::dom::HTMLCanvasElement>&
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::SetAsHTMLCanvasElement()
{
  if (mType == eHTMLCanvasElement) {
    return mValue.mHTMLCanvasElement.Value();
  }
  Uninit();
  mType = eHTMLCanvasElement;
  return mValue.mHTMLCanvasElement.SetValue();
}

bool
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::TrySetToHTMLCanvasElement(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::HTMLCanvasElement>& memberSlot = RawSetAsHTMLCanvasElement();
    {
      nsresult rv = UnwrapObject<prototypes::id::HTMLCanvasElement, mozilla::dom::HTMLCanvasElement>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyHTMLCanvasElement();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::DestroyHTMLCanvasElement()
{
  MOZ_ASSERT(IsHTMLCanvasElement(), "Wrong type!");
  mValue.mHTMLCanvasElement.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::HTMLVideoElement>&
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::RawSetAsHTMLVideoElement()
{
  if (mType == eHTMLVideoElement) {
    return mValue.mHTMLVideoElement.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eHTMLVideoElement;
  return mValue.mHTMLVideoElement.SetValue();
}

OwningNonNull<mozilla::dom::HTMLVideoElement>&
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::SetAsHTMLVideoElement()
{
  if (mType == eHTMLVideoElement) {
    return mValue.mHTMLVideoElement.Value();
  }
  Uninit();
  mType = eHTMLVideoElement;
  return mValue.mHTMLVideoElement.SetValue();
}

bool
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::TrySetToHTMLVideoElement(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::HTMLVideoElement>& memberSlot = RawSetAsHTMLVideoElement();
    {
      nsresult rv = UnwrapObject<prototypes::id::HTMLVideoElement, mozilla::dom::HTMLVideoElement>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyHTMLVideoElement();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::DestroyHTMLVideoElement()
{
  MOZ_ASSERT(IsHTMLVideoElement(), "Wrong type!");
  mValue.mHTMLVideoElement.Destroy();
  mType = eUninitialized;
}




void
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eHTMLImageElement: {
      DestroyHTMLImageElement();
      break;
    }
    case eHTMLCanvasElement: {
      DestroyHTMLCanvasElement();
      break;
    }
    case eHTMLVideoElement: {
      DestroyHTMLVideoElement();
      break;
    }
  }
}

bool
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eHTMLImageElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLImageElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eHTMLCanvasElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLCanvasElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eHTMLVideoElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLVideoElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::TraceUnion(JSTracer* trc)
{
}

void
OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement::operator=(const OwningHTMLImageElementOrHTMLCanvasElementOrHTMLVideoElement& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eHTMLImageElement: {
      SetAsHTMLImageElement() = aOther.GetAsHTMLImageElement();
      break;
    }
    case eHTMLCanvasElement: {
      SetAsHTMLCanvasElement() = aOther.GetAsHTMLCanvasElement();
      break;
    }
    case eHTMLVideoElement: {
      SetAsHTMLVideoElement() = aOther.GetAsHTMLVideoElement();
      break;
    }
  }
}

OwningNonNull<mozilla::dom::HTMLOptionElement>&
OwningHTMLOptionElementOrHTMLOptGroupElement::RawSetAsHTMLOptionElement()
{
  if (mType == eHTMLOptionElement) {
    return mValue.mHTMLOptionElement.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eHTMLOptionElement;
  return mValue.mHTMLOptionElement.SetValue();
}

OwningNonNull<mozilla::dom::HTMLOptionElement>&
OwningHTMLOptionElementOrHTMLOptGroupElement::SetAsHTMLOptionElement()
{
  if (mType == eHTMLOptionElement) {
    return mValue.mHTMLOptionElement.Value();
  }
  Uninit();
  mType = eHTMLOptionElement;
  return mValue.mHTMLOptionElement.SetValue();
}

bool
OwningHTMLOptionElementOrHTMLOptGroupElement::TrySetToHTMLOptionElement(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::HTMLOptionElement>& memberSlot = RawSetAsHTMLOptionElement();
    {
      nsresult rv = UnwrapObject<prototypes::id::HTMLOptionElement, mozilla::dom::HTMLOptionElement>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyHTMLOptionElement();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningHTMLOptionElementOrHTMLOptGroupElement::DestroyHTMLOptionElement()
{
  MOZ_ASSERT(IsHTMLOptionElement(), "Wrong type!");
  mValue.mHTMLOptionElement.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::HTMLOptGroupElement>&
OwningHTMLOptionElementOrHTMLOptGroupElement::RawSetAsHTMLOptGroupElement()
{
  if (mType == eHTMLOptGroupElement) {
    return mValue.mHTMLOptGroupElement.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eHTMLOptGroupElement;
  return mValue.mHTMLOptGroupElement.SetValue();
}

OwningNonNull<mozilla::dom::HTMLOptGroupElement>&
OwningHTMLOptionElementOrHTMLOptGroupElement::SetAsHTMLOptGroupElement()
{
  if (mType == eHTMLOptGroupElement) {
    return mValue.mHTMLOptGroupElement.Value();
  }
  Uninit();
  mType = eHTMLOptGroupElement;
  return mValue.mHTMLOptGroupElement.SetValue();
}

bool
OwningHTMLOptionElementOrHTMLOptGroupElement::TrySetToHTMLOptGroupElement(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::HTMLOptGroupElement>& memberSlot = RawSetAsHTMLOptGroupElement();
    {
      nsresult rv = UnwrapObject<prototypes::id::HTMLOptGroupElement, mozilla::dom::HTMLOptGroupElement>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyHTMLOptGroupElement();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningHTMLOptionElementOrHTMLOptGroupElement::DestroyHTMLOptGroupElement()
{
  MOZ_ASSERT(IsHTMLOptGroupElement(), "Wrong type!");
  mValue.mHTMLOptGroupElement.Destroy();
  mType = eUninitialized;
}




void
OwningHTMLOptionElementOrHTMLOptGroupElement::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eHTMLOptionElement: {
      DestroyHTMLOptionElement();
      break;
    }
    case eHTMLOptGroupElement: {
      DestroyHTMLOptGroupElement();
      break;
    }
  }
}

bool
OwningHTMLOptionElementOrHTMLOptGroupElement::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eHTMLOptionElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLOptionElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eHTMLOptGroupElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLOptGroupElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningHTMLOptionElementOrHTMLOptGroupElement::TraceUnion(JSTracer* trc)
{
}

void
OwningHTMLOptionElementOrHTMLOptGroupElement::operator=(const OwningHTMLOptionElementOrHTMLOptGroupElement& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eHTMLOptionElement: {
      SetAsHTMLOptionElement() = aOther.GetAsHTMLOptionElement();
      break;
    }
    case eHTMLOptGroupElement: {
      SetAsHTMLOptGroupElement() = aOther.GetAsHTMLOptGroupElement();
      break;
    }
  }
}

OwningNonNull<mozilla::dom::Headers>&
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::RawSetAsHeaders()
{
  if (mType == eHeaders) {
    return mValue.mHeaders.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eHeaders;
  return mValue.mHeaders.SetValue();
}

OwningNonNull<mozilla::dom::Headers>&
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::SetAsHeaders()
{
  if (mType == eHeaders) {
    return mValue.mHeaders.Value();
  }
  Uninit();
  mType = eHeaders;
  return mValue.mHeaders.SetValue();
}

bool
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::TrySetToHeaders(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::Headers>& memberSlot = RawSetAsHeaders();
    {
      nsresult rv = UnwrapObject<prototypes::id::Headers, mozilla::dom::Headers>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyHeaders();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::DestroyHeaders()
{
  MOZ_ASSERT(IsHeaders(), "Wrong type!");
  mValue.mHeaders.Destroy();
  mType = eUninitialized;
}




Sequence<Sequence<nsCString>>&
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::RawSetAsByteStringSequenceSequence()
{
  if (mType == eByteStringSequenceSequence) {
    return mValue.mByteStringSequenceSequence.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eByteStringSequenceSequence;
  return mValue.mByteStringSequenceSequence.SetValue();
}

Sequence<Sequence<nsCString>>&
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::SetAsByteStringSequenceSequence()
{
  if (mType == eByteStringSequenceSequence) {
    return mValue.mByteStringSequenceSequence.Value();
  }
  Uninit();
  mType = eByteStringSequenceSequence;
  return mValue.mByteStringSequenceSequence.SetValue();
}

bool
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::TrySetToByteStringSequenceSequence(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    Sequence<Sequence<nsCString>>& memberSlot = RawSetAsByteStringSequenceSequence();
    JS::ForOfIterator iter(cx);
    if (!iter.init(value, JS::ForOfIterator::AllowNonIterable)) {
      return false;
    }
    if (!iter.valueIsIterable()) {
      DestroyByteStringSequenceSequence();
      tryNext = true;
      return true;
    }
    Sequence<Sequence<nsCString>> &arr = memberSlot;
    JS::Rooted<JS::Value> temp(cx);
    while (true) {
      bool done;
      if (!iter.next(&temp, &done)) {
        return false;
      }
      if (done) {
        break;
      }
      Sequence<nsCString>* slotPtr = arr.AppendElement();
      if (!slotPtr) {
        JS_ReportOutOfMemory(cx);
        return false;
      }
      Sequence<nsCString>& slot = *slotPtr;
      if (temp.isObject()) {
        JS::ForOfIterator iter(cx);
        if (!iter.init(temp, JS::ForOfIterator::AllowNonIterable)) {
          return false;
        }
        if (!iter.valueIsIterable()) {
          ThrowErrorMessage(cx, MSG_NOT_SEQUENCE, "Element of member of HeadersOrByteStringSequenceSequenceOrByteStringMozMap");
          return false;
        }
        Sequence<nsCString> &arr = slot;
        JS::Rooted<JS::Value> temp(cx);
        while (true) {
          bool done;
          if (!iter.next(&temp, &done)) {
            return false;
          }
          if (done) {
            break;
          }
          nsCString* slotPtr = arr.AppendElement();
          if (!slotPtr) {
            JS_ReportOutOfMemory(cx);
            return false;
          }
          nsCString& slot = *slotPtr;
          if (!ConvertJSValueToByteString(cx, temp, false, slot)) {
            return false;
          }
        }
      } else {
        ThrowErrorMessage(cx, MSG_NOT_SEQUENCE, "Element of member of HeadersOrByteStringSequenceSequenceOrByteStringMozMap");
        return false;
      }
    }
  }
  return true;
}

void
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::DestroyByteStringSequenceSequence()
{
  MOZ_ASSERT(IsByteStringSequenceSequence(), "Wrong type!");
  mValue.mByteStringSequenceSequence.Destroy();
  mType = eUninitialized;
}




MozMap<nsCString>&
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::RawSetAsByteStringMozMap()
{
  if (mType == eByteStringMozMap) {
    return mValue.mByteStringMozMap.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eByteStringMozMap;
  return mValue.mByteStringMozMap.SetValue();
}

MozMap<nsCString>&
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::SetAsByteStringMozMap()
{
  if (mType == eByteStringMozMap) {
    return mValue.mByteStringMozMap.Value();
  }
  Uninit();
  mType = eByteStringMozMap;
  return mValue.mByteStringMozMap.SetValue();
}

bool
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::TrySetToByteStringMozMap(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    MozMap<nsCString>& memberSlot = RawSetAsByteStringMozMap();
    MozMap<nsCString> &mozMap = memberSlot;

    JS::Rooted<JSObject*> mozMapObj(cx, &value.toObject());
    JS::AutoIdArray ids(cx, JS_Enumerate(cx, mozMapObj));
    if (!ids) {
      return false;
    }
    JS::Rooted<JS::Value> propNameValue(cx);
    JS::Rooted<JS::Value> temp(cx);
    JS::Rooted<jsid> curId(cx);
    for (size_t i = 0; i < ids.length(); ++i) {
      // Make sure we get the value before converting the name, since
      // getting the value can trigger GC but our name is a dependent
      // string.
      curId = ids[i];
      binding_detail::FakeString propName;
      bool isSymbol;
      if (!ConvertIdToString(cx, curId, propName, isSymbol) ||
          (!isSymbol && !JS_GetPropertyById(cx, mozMapObj, curId, &temp))) {
        return false;
      }
      if (isSymbol) {
        continue;
      }

      nsCString* slotPtr = mozMap.AddEntry(propName);
      if (!slotPtr) {
        JS_ReportOutOfMemory(cx);
        return false;
      }
      nsCString& slot = *slotPtr;
      if (!ConvertJSValueToByteString(cx, temp, false, slot)) {
        return false;
      }
    }
  }
  return true;
}

void
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::DestroyByteStringMozMap()
{
  MOZ_ASSERT(IsByteStringMozMap(), "Wrong type!");
  mValue.mByteStringMozMap.Destroy();
  mType = eUninitialized;
}




void
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eHeaders: {
      DestroyHeaders();
      break;
    }
    case eByteStringSequenceSequence: {
      DestroyByteStringSequenceSequence();
      break;
    }
    case eByteStringMozMap: {
      DestroyByteStringMozMap();
      break;
    }
  }
}

bool
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eHeaders: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHeaders.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eByteStringSequenceSequence: {

      uint32_t length = mValue.mByteStringSequenceSequence.Value().Length();
      JS::Rooted<JSObject*> returnArray(cx, JS_NewArrayObject(cx, length));
      if (!returnArray) {
        return false;
      }
      // Scope for 'tmp'
      {
        JS::Rooted<JS::Value> tmp(cx);
        for (uint32_t sequenceIdx0 = 0; sequenceIdx0 < length; ++sequenceIdx0) {
          // Control block to let us common up the JS_DefineElement calls when there
          // are different ways to succeed at wrapping the object.
          do {

            uint32_t length = mValue.mByteStringSequenceSequence.Value()[sequenceIdx0].Length();
            JS::Rooted<JSObject*> returnArray(cx, JS_NewArrayObject(cx, length));
            if (!returnArray) {
              return false;
            }
            // Scope for 'tmp'
            {
              JS::Rooted<JS::Value> tmp(cx);
              for (uint32_t sequenceIdx1 = 0; sequenceIdx1 < length; ++sequenceIdx1) {
                // Control block to let us common up the JS_DefineElement calls when there
                // are different ways to succeed at wrapping the object.
                do {
                  if (!NonVoidByteStringToJsval(cx, mValue.mByteStringSequenceSequence.Value()[sequenceIdx0][sequenceIdx1], &tmp)) {
                    return false;
                  }
                  break;
                } while (0);
                if (!JS_DefineElement(cx, returnArray, sequenceIdx1, tmp,
                                      JSPROP_ENUMERATE)) {
                  return false;
                }
              }
            }
            tmp.setObject(*returnArray);
            break;
          } while (0);
          if (!JS_DefineElement(cx, returnArray, sequenceIdx0, tmp,
                                JSPROP_ENUMERATE)) {
            return false;
          }
        }
      }
      rval.setObject(*returnArray);
      return true;
      break;
    }
    case eByteStringMozMap: {

      nsTArray<nsString> keys;
      mValue.mByteStringMozMap.Value().GetKeys(keys);
      JS::Rooted<JSObject*> returnObj(cx, JS_NewObject(cx, nullptr, JS::NullPtr(), JS::NullPtr()));
      if (!returnObj) {
        return false;
      }
      // Scope for 'tmp'
      {
        JS::Rooted<JS::Value> tmp(cx);
        for (size_t idx = 0; idx < keys.Length(); ++idx) {
          auto& mozMapValue0 = mValue.mByteStringMozMap.Value().Get(keys[idx]);
          // Control block to let us common up the JS_DefineUCProperty calls when there
          // are different ways to succeed at wrapping the value.
          do {
            if (!NonVoidByteStringToJsval(cx, mozMapValue0, &tmp)) {
              return false;
            }
            break;
          } while (0);
          if (!JS_DefineUCProperty(cx, returnObj, keys[idx].get(),
                                   keys[idx].Length(), tmp,
                                   JSPROP_ENUMERATE)) {
            return false;
          }
        }
      }
      rval.setObject(*returnObj);
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningHeadersOrByteStringSequenceSequenceOrByteStringMozMap::TraceUnion(JSTracer* trc)
{
}

OwningNonNull<mozilla::dom::indexedDB::IDBObjectStore>&
OwningIDBObjectStoreOrIDBIndex::RawSetAsIDBObjectStore()
{
  if (mType == eIDBObjectStore) {
    return mValue.mIDBObjectStore.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eIDBObjectStore;
  return mValue.mIDBObjectStore.SetValue();
}

OwningNonNull<mozilla::dom::indexedDB::IDBObjectStore>&
OwningIDBObjectStoreOrIDBIndex::SetAsIDBObjectStore()
{
  if (mType == eIDBObjectStore) {
    return mValue.mIDBObjectStore.Value();
  }
  Uninit();
  mType = eIDBObjectStore;
  return mValue.mIDBObjectStore.SetValue();
}

bool
OwningIDBObjectStoreOrIDBIndex::TrySetToIDBObjectStore(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::indexedDB::IDBObjectStore>& memberSlot = RawSetAsIDBObjectStore();
    {
      nsresult rv = UnwrapObject<prototypes::id::IDBObjectStore, mozilla::dom::indexedDB::IDBObjectStore>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyIDBObjectStore();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningIDBObjectStoreOrIDBIndex::DestroyIDBObjectStore()
{
  MOZ_ASSERT(IsIDBObjectStore(), "Wrong type!");
  mValue.mIDBObjectStore.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::indexedDB::IDBIndex>&
OwningIDBObjectStoreOrIDBIndex::RawSetAsIDBIndex()
{
  if (mType == eIDBIndex) {
    return mValue.mIDBIndex.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eIDBIndex;
  return mValue.mIDBIndex.SetValue();
}

OwningNonNull<mozilla::dom::indexedDB::IDBIndex>&
OwningIDBObjectStoreOrIDBIndex::SetAsIDBIndex()
{
  if (mType == eIDBIndex) {
    return mValue.mIDBIndex.Value();
  }
  Uninit();
  mType = eIDBIndex;
  return mValue.mIDBIndex.SetValue();
}

bool
OwningIDBObjectStoreOrIDBIndex::TrySetToIDBIndex(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::indexedDB::IDBIndex>& memberSlot = RawSetAsIDBIndex();
    {
      nsresult rv = UnwrapObject<prototypes::id::IDBIndex, mozilla::dom::indexedDB::IDBIndex>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyIDBIndex();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningIDBObjectStoreOrIDBIndex::DestroyIDBIndex()
{
  MOZ_ASSERT(IsIDBIndex(), "Wrong type!");
  mValue.mIDBIndex.Destroy();
  mType = eUninitialized;
}




void
OwningIDBObjectStoreOrIDBIndex::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eIDBObjectStore: {
      DestroyIDBObjectStore();
      break;
    }
    case eIDBIndex: {
      DestroyIDBIndex();
      break;
    }
  }
}

bool
OwningIDBObjectStoreOrIDBIndex::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eIDBObjectStore: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mIDBObjectStore.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eIDBIndex: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mIDBIndex.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningIDBObjectStoreOrIDBIndex::TraceUnion(JSTracer* trc)
{
}

void
OwningIDBObjectStoreOrIDBIndex::operator=(const OwningIDBObjectStoreOrIDBIndex& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eIDBObjectStore: {
      SetAsIDBObjectStore() = aOther.GetAsIDBObjectStore();
      break;
    }
    case eIDBIndex: {
      SetAsIDBIndex() = aOther.GetAsIDBIndex();
      break;
    }
  }
}

OwningNonNull<mozilla::dom::indexedDB::IDBObjectStore>&
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::RawSetAsIDBObjectStore()
{
  if (mType == eIDBObjectStore) {
    return mValue.mIDBObjectStore.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eIDBObjectStore;
  return mValue.mIDBObjectStore.SetValue();
}

OwningNonNull<mozilla::dom::indexedDB::IDBObjectStore>&
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::SetAsIDBObjectStore()
{
  if (mType == eIDBObjectStore) {
    return mValue.mIDBObjectStore.Value();
  }
  Uninit();
  mType = eIDBObjectStore;
  return mValue.mIDBObjectStore.SetValue();
}

bool
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::TrySetToIDBObjectStore(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::indexedDB::IDBObjectStore>& memberSlot = RawSetAsIDBObjectStore();
    {
      nsresult rv = UnwrapObject<prototypes::id::IDBObjectStore, mozilla::dom::indexedDB::IDBObjectStore>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyIDBObjectStore();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::DestroyIDBObjectStore()
{
  MOZ_ASSERT(IsIDBObjectStore(), "Wrong type!");
  mValue.mIDBObjectStore.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::indexedDB::IDBIndex>&
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::RawSetAsIDBIndex()
{
  if (mType == eIDBIndex) {
    return mValue.mIDBIndex.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eIDBIndex;
  return mValue.mIDBIndex.SetValue();
}

OwningNonNull<mozilla::dom::indexedDB::IDBIndex>&
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::SetAsIDBIndex()
{
  if (mType == eIDBIndex) {
    return mValue.mIDBIndex.Value();
  }
  Uninit();
  mType = eIDBIndex;
  return mValue.mIDBIndex.SetValue();
}

bool
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::TrySetToIDBIndex(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::indexedDB::IDBIndex>& memberSlot = RawSetAsIDBIndex();
    {
      nsresult rv = UnwrapObject<prototypes::id::IDBIndex, mozilla::dom::indexedDB::IDBIndex>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyIDBIndex();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::DestroyIDBIndex()
{
  MOZ_ASSERT(IsIDBIndex(), "Wrong type!");
  mValue.mIDBIndex.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::indexedDB::IDBCursor>&
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::RawSetAsIDBCursor()
{
  if (mType == eIDBCursor) {
    return mValue.mIDBCursor.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eIDBCursor;
  return mValue.mIDBCursor.SetValue();
}

OwningNonNull<mozilla::dom::indexedDB::IDBCursor>&
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::SetAsIDBCursor()
{
  if (mType == eIDBCursor) {
    return mValue.mIDBCursor.Value();
  }
  Uninit();
  mType = eIDBCursor;
  return mValue.mIDBCursor.SetValue();
}

bool
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::TrySetToIDBCursor(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::indexedDB::IDBCursor>& memberSlot = RawSetAsIDBCursor();
    {
      nsresult rv = UnwrapObject<prototypes::id::IDBCursor, mozilla::dom::indexedDB::IDBCursor>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyIDBCursor();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::DestroyIDBCursor()
{
  MOZ_ASSERT(IsIDBCursor(), "Wrong type!");
  mValue.mIDBCursor.Destroy();
  mType = eUninitialized;
}




void
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eIDBObjectStore: {
      DestroyIDBObjectStore();
      break;
    }
    case eIDBIndex: {
      DestroyIDBIndex();
      break;
    }
    case eIDBCursor: {
      DestroyIDBCursor();
      break;
    }
  }
}

bool
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eIDBObjectStore: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mIDBObjectStore.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eIDBIndex: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mIDBIndex.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eIDBCursor: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mIDBCursor.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::TraceUnion(JSTracer* trc)
{
}

void
OwningIDBObjectStoreOrIDBIndexOrIDBCursor::operator=(const OwningIDBObjectStoreOrIDBIndexOrIDBCursor& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eIDBObjectStore: {
      SetAsIDBObjectStore() = aOther.GetAsIDBObjectStore();
      break;
    }
    case eIDBIndex: {
      SetAsIDBIndex() = aOther.GetAsIDBIndex();
      break;
    }
    case eIDBCursor: {
      SetAsIDBCursor() = aOther.GetAsIDBCursor();
      break;
    }
  }
}

int32_t&
OwningLongOrAutoKeyword::RawSetAsLong()
{
  if (mType == eLong) {
    return mValue.mLong.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eLong;
  return mValue.mLong.SetValue();
}

int32_t&
OwningLongOrAutoKeyword::SetAsLong()
{
  if (mType == eLong) {
    return mValue.mLong.Value();
  }
  Uninit();
  mType = eLong;
  return mValue.mLong.SetValue();
}

bool
OwningLongOrAutoKeyword::TrySetToLong(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    int32_t& memberSlot = RawSetAsLong();
    if (!ValueToPrimitive<int32_t, eDefault>(cx, value, &memberSlot)) {
      return false;
    }
  }
  return true;
}

void
OwningLongOrAutoKeyword::DestroyLong()
{
  MOZ_ASSERT(IsLong(), "Wrong type!");
  mValue.mLong.Destroy();
  mType = eUninitialized;
}




AutoKeyword&
OwningLongOrAutoKeyword::RawSetAsAutoKeyword()
{
  if (mType == eAutoKeyword) {
    return mValue.mAutoKeyword.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eAutoKeyword;
  return mValue.mAutoKeyword.SetValue();
}

AutoKeyword&
OwningLongOrAutoKeyword::SetAsAutoKeyword()
{
  if (mType == eAutoKeyword) {
    return mValue.mAutoKeyword.Value();
  }
  Uninit();
  mType = eAutoKeyword;
  return mValue.mAutoKeyword.SetValue();
}

bool
OwningLongOrAutoKeyword::TrySetToAutoKeyword(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    AutoKeyword& memberSlot = RawSetAsAutoKeyword();
    {
      bool ok;
      int index = FindEnumStringIndex<true>(cx, value, AutoKeywordValues::strings, "AutoKeyword", "Member of LongOrAutoKeyword", &ok);
      if (!ok) {
        return false;
      }
      MOZ_ASSERT(index >= 0);
      memberSlot = static_cast<AutoKeyword>(index);
    }
  }
  return true;
}

void
OwningLongOrAutoKeyword::DestroyAutoKeyword()
{
  MOZ_ASSERT(IsAutoKeyword(), "Wrong type!");
  mValue.mAutoKeyword.Destroy();
  mType = eUninitialized;
}




void
OwningLongOrAutoKeyword::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eLong: {
      DestroyLong();
      break;
    }
    case eAutoKeyword: {
      DestroyAutoKeyword();
      break;
    }
  }
}

bool
OwningLongOrAutoKeyword::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eLong: {
      rval.setInt32(int32_t(mValue.mLong.Value()));
      return true;
      break;
    }
    case eAutoKeyword: {
      {
        // Scope for resultStr
        MOZ_ASSERT(uint32_t(mValue.mAutoKeyword.Value()) < ArrayLength(AutoKeywordValues::strings));
        JSString* resultStr = JS_NewStringCopyN(cx, AutoKeywordValues::strings[uint32_t(mValue.mAutoKeyword.Value())].value, AutoKeywordValues::strings[uint32_t(mValue.mAutoKeyword.Value())].length);
        if (!resultStr) {
          return false;
        }
        rval.setString(resultStr);
        return true;
      }
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningLongOrAutoKeyword::TraceUnion(JSTracer* trc)
{
}

void
OwningLongOrAutoKeyword::operator=(const OwningLongOrAutoKeyword& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eLong: {
      SetAsLong() = aOther.GetAsLong();
      break;
    }
    case eAutoKeyword: {
      SetAsAutoKeyword() = aOther.GetAsAutoKeyword();
      break;
    }
  }
}

int32_t&
OwningLongOrMozSmsMessageOrMozMmsMessage::RawSetAsLong()
{
  if (mType == eLong) {
    return mValue.mLong.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eLong;
  return mValue.mLong.SetValue();
}

int32_t&
OwningLongOrMozSmsMessageOrMozMmsMessage::SetAsLong()
{
  if (mType == eLong) {
    return mValue.mLong.Value();
  }
  Uninit();
  mType = eLong;
  return mValue.mLong.SetValue();
}

bool
OwningLongOrMozSmsMessageOrMozMmsMessage::TrySetToLong(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    int32_t& memberSlot = RawSetAsLong();
    if (!ValueToPrimitive<int32_t, eDefault>(cx, value, &memberSlot)) {
      return false;
    }
  }
  return true;
}

void
OwningLongOrMozSmsMessageOrMozMmsMessage::DestroyLong()
{
  MOZ_ASSERT(IsLong(), "Wrong type!");
  mValue.mLong.Destroy();
  mType = eUninitialized;
}




nsRefPtr<nsIDOMMozSmsMessage>&
OwningLongOrMozSmsMessageOrMozMmsMessage::RawSetAsMozSmsMessage()
{
  if (mType == eMozSmsMessage) {
    return mValue.mMozSmsMessage.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eMozSmsMessage;
  return mValue.mMozSmsMessage.SetValue();
}

nsRefPtr<nsIDOMMozSmsMessage>&
OwningLongOrMozSmsMessageOrMozMmsMessage::SetAsMozSmsMessage()
{
  if (mType == eMozSmsMessage) {
    return mValue.mMozSmsMessage.Value();
  }
  Uninit();
  mType = eMozSmsMessage;
  return mValue.mMozSmsMessage.SetValue();
}

bool
OwningLongOrMozSmsMessageOrMozMmsMessage::TrySetToMozSmsMessage(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsRefPtr<nsIDOMMozSmsMessage>& memberSlot = RawSetAsMozSmsMessage();
    nsRefPtr<nsIDOMMozSmsMessage> mMozSmsMessageHolder;
    JS::Rooted<JS::Value> tmpVal(cx, value);
    nsIDOMMozSmsMessage* tmp;
    if (NS_FAILED(UnwrapArg<nsIDOMMozSmsMessage>(cx, value, &tmp, static_cast<nsIDOMMozSmsMessage**>(getter_AddRefs(mMozSmsMessageHolder)), &tmpVal))) {
      DestroyMozSmsMessage();
      tryNext = true;
      return true;
    }
    MOZ_ASSERT(tmp);
    memberSlot = tmp;
  }
  return true;
}

void
OwningLongOrMozSmsMessageOrMozMmsMessage::DestroyMozSmsMessage()
{
  MOZ_ASSERT(IsMozSmsMessage(), "Wrong type!");
  mValue.mMozSmsMessage.Destroy();
  mType = eUninitialized;
}




nsRefPtr<nsIDOMMozMmsMessage>&
OwningLongOrMozSmsMessageOrMozMmsMessage::RawSetAsMozMmsMessage()
{
  if (mType == eMozMmsMessage) {
    return mValue.mMozMmsMessage.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eMozMmsMessage;
  return mValue.mMozMmsMessage.SetValue();
}

nsRefPtr<nsIDOMMozMmsMessage>&
OwningLongOrMozSmsMessageOrMozMmsMessage::SetAsMozMmsMessage()
{
  if (mType == eMozMmsMessage) {
    return mValue.mMozMmsMessage.Value();
  }
  Uninit();
  mType = eMozMmsMessage;
  return mValue.mMozMmsMessage.SetValue();
}

bool
OwningLongOrMozSmsMessageOrMozMmsMessage::TrySetToMozMmsMessage(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsRefPtr<nsIDOMMozMmsMessage>& memberSlot = RawSetAsMozMmsMessage();
    nsRefPtr<nsIDOMMozMmsMessage> mMozMmsMessageHolder;
    JS::Rooted<JS::Value> tmpVal(cx, value);
    nsIDOMMozMmsMessage* tmp;
    if (NS_FAILED(UnwrapArg<nsIDOMMozMmsMessage>(cx, value, &tmp, static_cast<nsIDOMMozMmsMessage**>(getter_AddRefs(mMozMmsMessageHolder)), &tmpVal))) {
      DestroyMozMmsMessage();
      tryNext = true;
      return true;
    }
    MOZ_ASSERT(tmp);
    memberSlot = tmp;
  }
  return true;
}

void
OwningLongOrMozSmsMessageOrMozMmsMessage::DestroyMozMmsMessage()
{
  MOZ_ASSERT(IsMozMmsMessage(), "Wrong type!");
  mValue.mMozMmsMessage.Destroy();
  mType = eUninitialized;
}




void
OwningLongOrMozSmsMessageOrMozMmsMessage::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eLong: {
      DestroyLong();
      break;
    }
    case eMozSmsMessage: {
      DestroyMozSmsMessage();
      break;
    }
    case eMozMmsMessage: {
      DestroyMozMmsMessage();
      break;
    }
  }
}

bool
OwningLongOrMozSmsMessageOrMozMmsMessage::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eLong: {
      rval.setInt32(int32_t(mValue.mLong.Value()));
      return true;
      break;
    }
    case eMozSmsMessage: {
      if (!WrapObject(cx, mValue.mMozSmsMessage.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eMozMmsMessage: {
      if (!WrapObject(cx, mValue.mMozMmsMessage.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningLongOrMozSmsMessageOrMozMmsMessage::TraceUnion(JSTracer* trc)
{
}

void
OwningLongOrMozSmsMessageOrMozMmsMessage::operator=(const OwningLongOrMozSmsMessageOrMozMmsMessage& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eLong: {
      SetAsLong() = aOther.GetAsLong();
      break;
    }
    case eMozSmsMessage: {
      SetAsMozSmsMessage() = aOther.GetAsMozSmsMessage();
      break;
    }
    case eMozMmsMessage: {
      SetAsMozMmsMessage() = aOther.GetAsMozMmsMessage();
      break;
    }
  }
}

nsRefPtr<nsIDOMMozSmsMessage>&
OwningMozSmsMessageOrMozMmsMessage::RawSetAsMozSmsMessage()
{
  if (mType == eMozSmsMessage) {
    return mValue.mMozSmsMessage.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eMozSmsMessage;
  return mValue.mMozSmsMessage.SetValue();
}

nsRefPtr<nsIDOMMozSmsMessage>&
OwningMozSmsMessageOrMozMmsMessage::SetAsMozSmsMessage()
{
  if (mType == eMozSmsMessage) {
    return mValue.mMozSmsMessage.Value();
  }
  Uninit();
  mType = eMozSmsMessage;
  return mValue.mMozSmsMessage.SetValue();
}

bool
OwningMozSmsMessageOrMozMmsMessage::TrySetToMozSmsMessage(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsRefPtr<nsIDOMMozSmsMessage>& memberSlot = RawSetAsMozSmsMessage();
    nsRefPtr<nsIDOMMozSmsMessage> mMozSmsMessageHolder;
    JS::Rooted<JS::Value> tmpVal(cx, value);
    nsIDOMMozSmsMessage* tmp;
    if (NS_FAILED(UnwrapArg<nsIDOMMozSmsMessage>(cx, value, &tmp, static_cast<nsIDOMMozSmsMessage**>(getter_AddRefs(mMozSmsMessageHolder)), &tmpVal))) {
      DestroyMozSmsMessage();
      tryNext = true;
      return true;
    }
    MOZ_ASSERT(tmp);
    memberSlot = tmp;
  }
  return true;
}

void
OwningMozSmsMessageOrMozMmsMessage::DestroyMozSmsMessage()
{
  MOZ_ASSERT(IsMozSmsMessage(), "Wrong type!");
  mValue.mMozSmsMessage.Destroy();
  mType = eUninitialized;
}




nsRefPtr<nsIDOMMozMmsMessage>&
OwningMozSmsMessageOrMozMmsMessage::RawSetAsMozMmsMessage()
{
  if (mType == eMozMmsMessage) {
    return mValue.mMozMmsMessage.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eMozMmsMessage;
  return mValue.mMozMmsMessage.SetValue();
}

nsRefPtr<nsIDOMMozMmsMessage>&
OwningMozSmsMessageOrMozMmsMessage::SetAsMozMmsMessage()
{
  if (mType == eMozMmsMessage) {
    return mValue.mMozMmsMessage.Value();
  }
  Uninit();
  mType = eMozMmsMessage;
  return mValue.mMozMmsMessage.SetValue();
}

bool
OwningMozSmsMessageOrMozMmsMessage::TrySetToMozMmsMessage(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsRefPtr<nsIDOMMozMmsMessage>& memberSlot = RawSetAsMozMmsMessage();
    nsRefPtr<nsIDOMMozMmsMessage> mMozMmsMessageHolder;
    JS::Rooted<JS::Value> tmpVal(cx, value);
    nsIDOMMozMmsMessage* tmp;
    if (NS_FAILED(UnwrapArg<nsIDOMMozMmsMessage>(cx, value, &tmp, static_cast<nsIDOMMozMmsMessage**>(getter_AddRefs(mMozMmsMessageHolder)), &tmpVal))) {
      DestroyMozMmsMessage();
      tryNext = true;
      return true;
    }
    MOZ_ASSERT(tmp);
    memberSlot = tmp;
  }
  return true;
}

void
OwningMozSmsMessageOrMozMmsMessage::DestroyMozMmsMessage()
{
  MOZ_ASSERT(IsMozMmsMessage(), "Wrong type!");
  mValue.mMozMmsMessage.Destroy();
  mType = eUninitialized;
}




void
OwningMozSmsMessageOrMozMmsMessage::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eMozSmsMessage: {
      DestroyMozSmsMessage();
      break;
    }
    case eMozMmsMessage: {
      DestroyMozMmsMessage();
      break;
    }
  }
}

bool
OwningMozSmsMessageOrMozMmsMessage::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eMozSmsMessage: {
      if (!WrapObject(cx, mValue.mMozSmsMessage.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eMozMmsMessage: {
      if (!WrapObject(cx, mValue.mMozMmsMessage.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningMozSmsMessageOrMozMmsMessage::TraceUnion(JSTracer* trc)
{
}

void
OwningMozSmsMessageOrMozMmsMessage::operator=(const OwningMozSmsMessageOrMozMmsMessage& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eMozSmsMessage: {
      SetAsMozSmsMessage() = aOther.GetAsMozSmsMessage();
      break;
    }
    case eMozMmsMessage: {
      SetAsMozMmsMessage() = aOther.GetAsMozMmsMessage();
      break;
    }
  }
}

OwningNonNull<mozilla::dom::NetworkStatsData>&
OwningNetworkStatsDataOrPowerStatsData::RawSetAsNetworkStatsData()
{
  if (mType == eNetworkStatsData) {
    return mValue.mNetworkStatsData.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eNetworkStatsData;
  return mValue.mNetworkStatsData.SetValue();
}

OwningNonNull<mozilla::dom::NetworkStatsData>&
OwningNetworkStatsDataOrPowerStatsData::SetAsNetworkStatsData()
{
  if (mType == eNetworkStatsData) {
    return mValue.mNetworkStatsData.Value();
  }
  Uninit();
  mType = eNetworkStatsData;
  return mValue.mNetworkStatsData.SetValue();
}

bool
OwningNetworkStatsDataOrPowerStatsData::TrySetToNetworkStatsData(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::NetworkStatsData>& memberSlot = RawSetAsNetworkStatsData();
    {
      nsresult rv = UnwrapObject<prototypes::id::NetworkStatsData, mozilla::dom::NetworkStatsData>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyNetworkStatsData();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningNetworkStatsDataOrPowerStatsData::DestroyNetworkStatsData()
{
  MOZ_ASSERT(IsNetworkStatsData(), "Wrong type!");
  mValue.mNetworkStatsData.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::PowerStatsData>&
OwningNetworkStatsDataOrPowerStatsData::RawSetAsPowerStatsData()
{
  if (mType == ePowerStatsData) {
    return mValue.mPowerStatsData.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = ePowerStatsData;
  return mValue.mPowerStatsData.SetValue();
}

OwningNonNull<mozilla::dom::PowerStatsData>&
OwningNetworkStatsDataOrPowerStatsData::SetAsPowerStatsData()
{
  if (mType == ePowerStatsData) {
    return mValue.mPowerStatsData.Value();
  }
  Uninit();
  mType = ePowerStatsData;
  return mValue.mPowerStatsData.SetValue();
}

bool
OwningNetworkStatsDataOrPowerStatsData::TrySetToPowerStatsData(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::PowerStatsData>& memberSlot = RawSetAsPowerStatsData();
    {
      nsresult rv = UnwrapObject<prototypes::id::PowerStatsData, mozilla::dom::PowerStatsData>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyPowerStatsData();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningNetworkStatsDataOrPowerStatsData::DestroyPowerStatsData()
{
  MOZ_ASSERT(IsPowerStatsData(), "Wrong type!");
  mValue.mPowerStatsData.Destroy();
  mType = eUninitialized;
}




void
OwningNetworkStatsDataOrPowerStatsData::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eNetworkStatsData: {
      DestroyNetworkStatsData();
      break;
    }
    case ePowerStatsData: {
      DestroyPowerStatsData();
      break;
    }
  }
}

bool
OwningNetworkStatsDataOrPowerStatsData::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eNetworkStatsData: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mNetworkStatsData.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case ePowerStatsData: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mPowerStatsData.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningNetworkStatsDataOrPowerStatsData::TraceUnion(JSTracer* trc)
{
}

void
OwningNetworkStatsDataOrPowerStatsData::operator=(const OwningNetworkStatsDataOrPowerStatsData& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eNetworkStatsData: {
      SetAsNetworkStatsData() = aOther.GetAsNetworkStatsData();
      break;
    }
    case ePowerStatsData: {
      SetAsPowerStatsData() = aOther.GetAsPowerStatsData();
      break;
    }
  }
}

OwningNonNull<nsINode>&
OwningNodeOrHTMLCollection::RawSetAsNode()
{
  if (mType == eNode) {
    return mValue.mNode.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eNode;
  return mValue.mNode.SetValue();
}

OwningNonNull<nsINode>&
OwningNodeOrHTMLCollection::SetAsNode()
{
  if (mType == eNode) {
    return mValue.mNode.Value();
  }
  Uninit();
  mType = eNode;
  return mValue.mNode.SetValue();
}

bool
OwningNodeOrHTMLCollection::TrySetToNode(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<nsINode>& memberSlot = RawSetAsNode();
    {
      nsresult rv = UnwrapObject<prototypes::id::Node, nsINode>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyNode();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningNodeOrHTMLCollection::DestroyNode()
{
  MOZ_ASSERT(IsNode(), "Wrong type!");
  mValue.mNode.Destroy();
  mType = eUninitialized;
}




OwningNonNull<nsIHTMLCollection>&
OwningNodeOrHTMLCollection::RawSetAsHTMLCollection()
{
  if (mType == eHTMLCollection) {
    return mValue.mHTMLCollection.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eHTMLCollection;
  return mValue.mHTMLCollection.SetValue();
}

OwningNonNull<nsIHTMLCollection>&
OwningNodeOrHTMLCollection::SetAsHTMLCollection()
{
  if (mType == eHTMLCollection) {
    return mValue.mHTMLCollection.Value();
  }
  Uninit();
  mType = eHTMLCollection;
  return mValue.mHTMLCollection.SetValue();
}

bool
OwningNodeOrHTMLCollection::TrySetToHTMLCollection(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<nsIHTMLCollection>& memberSlot = RawSetAsHTMLCollection();
    {
      nsresult rv = UnwrapObject<prototypes::id::HTMLCollection, nsIHTMLCollection>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyHTMLCollection();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningNodeOrHTMLCollection::DestroyHTMLCollection()
{
  MOZ_ASSERT(IsHTMLCollection(), "Wrong type!");
  mValue.mHTMLCollection.Destroy();
  mType = eUninitialized;
}




void
OwningNodeOrHTMLCollection::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eNode: {
      DestroyNode();
      break;
    }
    case eHTMLCollection: {
      DestroyHTMLCollection();
      break;
    }
  }
}

bool
OwningNodeOrHTMLCollection::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eNode: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mNode.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eHTMLCollection: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mHTMLCollection.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningNodeOrHTMLCollection::TraceUnion(JSTracer* trc)
{
}

void
OwningNodeOrHTMLCollection::operator=(const OwningNodeOrHTMLCollection& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eNode: {
      SetAsNode() = aOther.GetAsNode();
      break;
    }
    case eHTMLCollection: {
      SetAsHTMLCollection() = aOther.GetAsHTMLCollection();
      break;
    }
  }
}

JSObject*&
OwningObjectOrString::RawSetAsObject()
{
  if (mType == eObject) {
    return mValue.mObject.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eObject;
  return mValue.mObject.SetValue();
}

JSObject*&
OwningObjectOrString::SetAsObject()
{
  if (mType == eObject) {
    return mValue.mObject.Value();
  }
  Uninit();
  mType = eObject;
  return mValue.mObject.SetValue();
}


void
OwningObjectOrString::DestroyObject()
{
  MOZ_ASSERT(IsObject(), "Wrong type!");
  mValue.mObject.Destroy();
  mType = eUninitialized;
}




nsString&
OwningObjectOrString::RawSetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eString;
  return mValue.mString.SetValue();
}

nsString&
OwningObjectOrString::SetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  Uninit();
  mType = eString;
  return mValue.mString.SetValue();
}

bool
OwningObjectOrString::TrySetToString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
  }
  return true;
}


void
OwningObjectOrString::DestroyString()
{
  MOZ_ASSERT(IsString(), "Wrong type!");
  mValue.mString.Destroy();
  mType = eUninitialized;
}




void
OwningObjectOrString::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eObject: {
      DestroyObject();
      break;
    }
    case eString: {
      DestroyString();
      break;
    }
  }
}

bool
OwningObjectOrString::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eObject: {
      JS::ExposeObjectToActiveJS(mValue.mObject.Value());
      rval.setObject(*mValue.mObject.Value());
      if (!MaybeWrapObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningObjectOrString::TraceUnion(JSTracer* trc)
{
  switch (mType) {
    case eObject: {
      JS_CallUnbarrieredObjectTracer(trc, &mValue.mObject.Value(), "mValue.mObject");
      break;
    }
    default: {
      break;
    }
  }
}

OwningNonNull<mozilla::dom::RadioNodeList>&
OwningRadioNodeListOrElement::RawSetAsRadioNodeList()
{
  if (mType == eRadioNodeList) {
    return mValue.mRadioNodeList.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eRadioNodeList;
  return mValue.mRadioNodeList.SetValue();
}

OwningNonNull<mozilla::dom::RadioNodeList>&
OwningRadioNodeListOrElement::SetAsRadioNodeList()
{
  if (mType == eRadioNodeList) {
    return mValue.mRadioNodeList.Value();
  }
  Uninit();
  mType = eRadioNodeList;
  return mValue.mRadioNodeList.SetValue();
}

bool
OwningRadioNodeListOrElement::TrySetToRadioNodeList(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::RadioNodeList>& memberSlot = RawSetAsRadioNodeList();
    {
      nsresult rv = UnwrapObject<prototypes::id::RadioNodeList, mozilla::dom::RadioNodeList>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyRadioNodeList();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningRadioNodeListOrElement::DestroyRadioNodeList()
{
  MOZ_ASSERT(IsRadioNodeList(), "Wrong type!");
  mValue.mRadioNodeList.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::Element>&
OwningRadioNodeListOrElement::RawSetAsElement()
{
  if (mType == eElement) {
    return mValue.mElement.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eElement;
  return mValue.mElement.SetValue();
}

OwningNonNull<mozilla::dom::Element>&
OwningRadioNodeListOrElement::SetAsElement()
{
  if (mType == eElement) {
    return mValue.mElement.Value();
  }
  Uninit();
  mType = eElement;
  return mValue.mElement.SetValue();
}

bool
OwningRadioNodeListOrElement::TrySetToElement(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::Element>& memberSlot = RawSetAsElement();
    {
      nsresult rv = UnwrapObject<prototypes::id::Element, mozilla::dom::Element>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyElement();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningRadioNodeListOrElement::DestroyElement()
{
  MOZ_ASSERT(IsElement(), "Wrong type!");
  mValue.mElement.Destroy();
  mType = eUninitialized;
}




void
OwningRadioNodeListOrElement::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eRadioNodeList: {
      DestroyRadioNodeList();
      break;
    }
    case eElement: {
      DestroyElement();
      break;
    }
  }
}

bool
OwningRadioNodeListOrElement::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eRadioNodeList: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mRadioNodeList.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningRadioNodeListOrElement::TraceUnion(JSTracer* trc)
{
}

void
OwningRadioNodeListOrElement::operator=(const OwningRadioNodeListOrElement& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eRadioNodeList: {
      SetAsRadioNodeList() = aOther.GetAsRadioNodeList();
      break;
    }
    case eElement: {
      SetAsElement() = aOther.GetAsElement();
      break;
    }
  }
}

OwningNonNull<mozilla::dom::Request>&
OwningRequestOrScalarValueString::RawSetAsRequest()
{
  if (mType == eRequest) {
    return mValue.mRequest.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eRequest;
  return mValue.mRequest.SetValue();
}

OwningNonNull<mozilla::dom::Request>&
OwningRequestOrScalarValueString::SetAsRequest()
{
  if (mType == eRequest) {
    return mValue.mRequest.Value();
  }
  Uninit();
  mType = eRequest;
  return mValue.mRequest.SetValue();
}

bool
OwningRequestOrScalarValueString::TrySetToRequest(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::Request>& memberSlot = RawSetAsRequest();
    {
      nsresult rv = UnwrapObject<prototypes::id::Request, mozilla::dom::Request>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyRequest();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningRequestOrScalarValueString::DestroyRequest()
{
  MOZ_ASSERT(IsRequest(), "Wrong type!");
  mValue.mRequest.Destroy();
  mType = eUninitialized;
}




nsString&
OwningRequestOrScalarValueString::RawSetAsScalarValueString()
{
  if (mType == eScalarValueString) {
    return mValue.mScalarValueString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eScalarValueString;
  return mValue.mScalarValueString.SetValue();
}

nsString&
OwningRequestOrScalarValueString::SetAsScalarValueString()
{
  if (mType == eScalarValueString) {
    return mValue.mScalarValueString.Value();
  }
  Uninit();
  mType = eScalarValueString;
  return mValue.mScalarValueString.SetValue();
}

bool
OwningRequestOrScalarValueString::TrySetToScalarValueString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsScalarValueString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
    NormalizeScalarValueString(cx, memberSlot);
  }
  return true;
}


void
OwningRequestOrScalarValueString::DestroyScalarValueString()
{
  MOZ_ASSERT(IsScalarValueString(), "Wrong type!");
  mValue.mScalarValueString.Destroy();
  mType = eUninitialized;
}




void
OwningRequestOrScalarValueString::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eRequest: {
      DestroyRequest();
      break;
    }
    case eScalarValueString: {
      DestroyScalarValueString();
      break;
    }
  }
}

bool
OwningRequestOrScalarValueString::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eRequest: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mRequest.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eScalarValueString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mScalarValueString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningRequestOrScalarValueString::TraceUnion(JSTracer* trc)
{
}

void
OwningRequestOrScalarValueString::operator=(const OwningRequestOrScalarValueString& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eRequest: {
      SetAsRequest() = aOther.GetAsRequest();
      break;
    }
    case eScalarValueString: {
      SetAsScalarValueString() = aOther.GetAsScalarValueString();
      break;
    }
  }
}

nsString&
OwningStringOrArrayBufferOrArrayBufferView::RawSetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eString;
  return mValue.mString.SetValue();
}

nsString&
OwningStringOrArrayBufferOrArrayBufferView::SetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  Uninit();
  mType = eString;
  return mValue.mString.SetValue();
}

bool
OwningStringOrArrayBufferOrArrayBufferView::TrySetToString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
  }
  return true;
}


void
OwningStringOrArrayBufferOrArrayBufferView::DestroyString()
{
  MOZ_ASSERT(IsString(), "Wrong type!");
  mValue.mString.Destroy();
  mType = eUninitialized;
}




ArrayBuffer&
OwningStringOrArrayBufferOrArrayBufferView::RawSetAsArrayBuffer()
{
  if (mType == eArrayBuffer) {
    return mValue.mArrayBuffer.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eArrayBuffer;
  return mValue.mArrayBuffer.SetValue();
}

ArrayBuffer&
OwningStringOrArrayBufferOrArrayBufferView::SetAsArrayBuffer()
{
  if (mType == eArrayBuffer) {
    return mValue.mArrayBuffer.Value();
  }
  Uninit();
  mType = eArrayBuffer;
  return mValue.mArrayBuffer.SetValue();
}

bool
OwningStringOrArrayBufferOrArrayBufferView::TrySetToArrayBuffer(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    ArrayBuffer& memberSlot = RawSetAsArrayBuffer();
    if (!memberSlot.Init(&value.toObject())) {
      DestroyArrayBuffer();
      tryNext = true;
      return true;
    }
  }
  return true;
}

void
OwningStringOrArrayBufferOrArrayBufferView::DestroyArrayBuffer()
{
  MOZ_ASSERT(IsArrayBuffer(), "Wrong type!");
  mValue.mArrayBuffer.Destroy();
  mType = eUninitialized;
}




ArrayBufferView&
OwningStringOrArrayBufferOrArrayBufferView::RawSetAsArrayBufferView()
{
  if (mType == eArrayBufferView) {
    return mValue.mArrayBufferView.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eArrayBufferView;
  return mValue.mArrayBufferView.SetValue();
}

ArrayBufferView&
OwningStringOrArrayBufferOrArrayBufferView::SetAsArrayBufferView()
{
  if (mType == eArrayBufferView) {
    return mValue.mArrayBufferView.Value();
  }
  Uninit();
  mType = eArrayBufferView;
  return mValue.mArrayBufferView.SetValue();
}

bool
OwningStringOrArrayBufferOrArrayBufferView::TrySetToArrayBufferView(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    ArrayBufferView& memberSlot = RawSetAsArrayBufferView();
    if (!memberSlot.Init(&value.toObject())) {
      DestroyArrayBufferView();
      tryNext = true;
      return true;
    }
  }
  return true;
}

void
OwningStringOrArrayBufferOrArrayBufferView::DestroyArrayBufferView()
{
  MOZ_ASSERT(IsArrayBufferView(), "Wrong type!");
  mValue.mArrayBufferView.Destroy();
  mType = eUninitialized;
}




void
OwningStringOrArrayBufferOrArrayBufferView::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eString: {
      DestroyString();
      break;
    }
    case eArrayBuffer: {
      DestroyArrayBuffer();
      break;
    }
    case eArrayBufferView: {
      DestroyArrayBufferView();
      break;
    }
  }
}

bool
OwningStringOrArrayBufferOrArrayBufferView::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eArrayBuffer: {
      rval.setObject(*mValue.mArrayBuffer.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eArrayBufferView: {
      rval.setObject(*mValue.mArrayBufferView.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningStringOrArrayBufferOrArrayBufferView::TraceUnion(JSTracer* trc)
{
  switch (mType) {
    case eArrayBuffer: {
      mValue.mArrayBuffer.Value().TraceSelf(trc);
      break;
    }
    case eArrayBufferView: {
      mValue.mArrayBufferView.Value().TraceSelf(trc);
      break;
    }
    default: {
      break;
    }
  }
}

nsString&
OwningStringOrBlobOrArrayBufferOrArrayBufferView::RawSetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eString;
  return mValue.mString.SetValue();
}

nsString&
OwningStringOrBlobOrArrayBufferOrArrayBufferView::SetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  Uninit();
  mType = eString;
  return mValue.mString.SetValue();
}

bool
OwningStringOrBlobOrArrayBufferOrArrayBufferView::TrySetToString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
  }
  return true;
}


void
OwningStringOrBlobOrArrayBufferOrArrayBufferView::DestroyString()
{
  MOZ_ASSERT(IsString(), "Wrong type!");
  mValue.mString.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::File>&
OwningStringOrBlobOrArrayBufferOrArrayBufferView::RawSetAsBlob()
{
  if (mType == eBlob) {
    return mValue.mBlob.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eBlob;
  return mValue.mBlob.SetValue();
}

OwningNonNull<mozilla::dom::File>&
OwningStringOrBlobOrArrayBufferOrArrayBufferView::SetAsBlob()
{
  if (mType == eBlob) {
    return mValue.mBlob.Value();
  }
  Uninit();
  mType = eBlob;
  return mValue.mBlob.SetValue();
}

bool
OwningStringOrBlobOrArrayBufferOrArrayBufferView::TrySetToBlob(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::File>& memberSlot = RawSetAsBlob();
    {
      nsresult rv = UnwrapObject<prototypes::id::Blob, mozilla::dom::File>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyBlob();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningStringOrBlobOrArrayBufferOrArrayBufferView::DestroyBlob()
{
  MOZ_ASSERT(IsBlob(), "Wrong type!");
  mValue.mBlob.Destroy();
  mType = eUninitialized;
}




ArrayBuffer&
OwningStringOrBlobOrArrayBufferOrArrayBufferView::RawSetAsArrayBuffer()
{
  if (mType == eArrayBuffer) {
    return mValue.mArrayBuffer.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eArrayBuffer;
  return mValue.mArrayBuffer.SetValue();
}

ArrayBuffer&
OwningStringOrBlobOrArrayBufferOrArrayBufferView::SetAsArrayBuffer()
{
  if (mType == eArrayBuffer) {
    return mValue.mArrayBuffer.Value();
  }
  Uninit();
  mType = eArrayBuffer;
  return mValue.mArrayBuffer.SetValue();
}

bool
OwningStringOrBlobOrArrayBufferOrArrayBufferView::TrySetToArrayBuffer(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    ArrayBuffer& memberSlot = RawSetAsArrayBuffer();
    if (!memberSlot.Init(&value.toObject())) {
      DestroyArrayBuffer();
      tryNext = true;
      return true;
    }
  }
  return true;
}

void
OwningStringOrBlobOrArrayBufferOrArrayBufferView::DestroyArrayBuffer()
{
  MOZ_ASSERT(IsArrayBuffer(), "Wrong type!");
  mValue.mArrayBuffer.Destroy();
  mType = eUninitialized;
}




ArrayBufferView&
OwningStringOrBlobOrArrayBufferOrArrayBufferView::RawSetAsArrayBufferView()
{
  if (mType == eArrayBufferView) {
    return mValue.mArrayBufferView.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eArrayBufferView;
  return mValue.mArrayBufferView.SetValue();
}

ArrayBufferView&
OwningStringOrBlobOrArrayBufferOrArrayBufferView::SetAsArrayBufferView()
{
  if (mType == eArrayBufferView) {
    return mValue.mArrayBufferView.Value();
  }
  Uninit();
  mType = eArrayBufferView;
  return mValue.mArrayBufferView.SetValue();
}

bool
OwningStringOrBlobOrArrayBufferOrArrayBufferView::TrySetToArrayBufferView(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    ArrayBufferView& memberSlot = RawSetAsArrayBufferView();
    if (!memberSlot.Init(&value.toObject())) {
      DestroyArrayBufferView();
      tryNext = true;
      return true;
    }
  }
  return true;
}

void
OwningStringOrBlobOrArrayBufferOrArrayBufferView::DestroyArrayBufferView()
{
  MOZ_ASSERT(IsArrayBufferView(), "Wrong type!");
  mValue.mArrayBufferView.Destroy();
  mType = eUninitialized;
}




void
OwningStringOrBlobOrArrayBufferOrArrayBufferView::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eString: {
      DestroyString();
      break;
    }
    case eBlob: {
      DestroyBlob();
      break;
    }
    case eArrayBuffer: {
      DestroyArrayBuffer();
      break;
    }
    case eArrayBufferView: {
      DestroyArrayBufferView();
      break;
    }
  }
}

bool
OwningStringOrBlobOrArrayBufferOrArrayBufferView::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eBlob: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mBlob.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eArrayBuffer: {
      rval.setObject(*mValue.mArrayBuffer.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    case eArrayBufferView: {
      rval.setObject(*mValue.mArrayBufferView.Value().Obj());
      if (!MaybeWrapNonDOMObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningStringOrBlobOrArrayBufferOrArrayBufferView::TraceUnion(JSTracer* trc)
{
  switch (mType) {
    case eArrayBuffer: {
      mValue.mArrayBuffer.Value().TraceSelf(trc);
      break;
    }
    case eArrayBufferView: {
      mValue.mArrayBufferView.Value().TraceSelf(trc);
      break;
    }
    default: {
      break;
    }
  }
}

nsString&
OwningStringOrCanvasGradientOrCanvasPattern::RawSetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eString;
  return mValue.mString.SetValue();
}

nsString&
OwningStringOrCanvasGradientOrCanvasPattern::SetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  Uninit();
  mType = eString;
  return mValue.mString.SetValue();
}

bool
OwningStringOrCanvasGradientOrCanvasPattern::TrySetToString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
  }
  return true;
}


void
OwningStringOrCanvasGradientOrCanvasPattern::DestroyString()
{
  MOZ_ASSERT(IsString(), "Wrong type!");
  mValue.mString.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::CanvasGradient>&
OwningStringOrCanvasGradientOrCanvasPattern::RawSetAsCanvasGradient()
{
  if (mType == eCanvasGradient) {
    return mValue.mCanvasGradient.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eCanvasGradient;
  return mValue.mCanvasGradient.SetValue();
}

OwningNonNull<mozilla::dom::CanvasGradient>&
OwningStringOrCanvasGradientOrCanvasPattern::SetAsCanvasGradient()
{
  if (mType == eCanvasGradient) {
    return mValue.mCanvasGradient.Value();
  }
  Uninit();
  mType = eCanvasGradient;
  return mValue.mCanvasGradient.SetValue();
}

bool
OwningStringOrCanvasGradientOrCanvasPattern::TrySetToCanvasGradient(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::CanvasGradient>& memberSlot = RawSetAsCanvasGradient();
    {
      nsresult rv = UnwrapObject<prototypes::id::CanvasGradient, mozilla::dom::CanvasGradient>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyCanvasGradient();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningStringOrCanvasGradientOrCanvasPattern::DestroyCanvasGradient()
{
  MOZ_ASSERT(IsCanvasGradient(), "Wrong type!");
  mValue.mCanvasGradient.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::CanvasPattern>&
OwningStringOrCanvasGradientOrCanvasPattern::RawSetAsCanvasPattern()
{
  if (mType == eCanvasPattern) {
    return mValue.mCanvasPattern.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eCanvasPattern;
  return mValue.mCanvasPattern.SetValue();
}

OwningNonNull<mozilla::dom::CanvasPattern>&
OwningStringOrCanvasGradientOrCanvasPattern::SetAsCanvasPattern()
{
  if (mType == eCanvasPattern) {
    return mValue.mCanvasPattern.Value();
  }
  Uninit();
  mType = eCanvasPattern;
  return mValue.mCanvasPattern.SetValue();
}

bool
OwningStringOrCanvasGradientOrCanvasPattern::TrySetToCanvasPattern(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::CanvasPattern>& memberSlot = RawSetAsCanvasPattern();
    {
      nsresult rv = UnwrapObject<prototypes::id::CanvasPattern, mozilla::dom::CanvasPattern>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyCanvasPattern();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningStringOrCanvasGradientOrCanvasPattern::DestroyCanvasPattern()
{
  MOZ_ASSERT(IsCanvasPattern(), "Wrong type!");
  mValue.mCanvasPattern.Destroy();
  mType = eUninitialized;
}




void
OwningStringOrCanvasGradientOrCanvasPattern::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eString: {
      DestroyString();
      break;
    }
    case eCanvasGradient: {
      DestroyCanvasGradient();
      break;
    }
    case eCanvasPattern: {
      DestroyCanvasPattern();
      break;
    }
  }
}

bool
OwningStringOrCanvasGradientOrCanvasPattern::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eCanvasGradient: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mCanvasGradient.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eCanvasPattern: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mCanvasPattern.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningStringOrCanvasGradientOrCanvasPattern::TraceUnion(JSTracer* trc)
{
}

void
OwningStringOrCanvasGradientOrCanvasPattern::operator=(const OwningStringOrCanvasGradientOrCanvasPattern& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eString: {
      SetAsString() = aOther.GetAsString();
      break;
    }
    case eCanvasGradient: {
      SetAsCanvasGradient() = aOther.GetAsCanvasGradient();
      break;
    }
    case eCanvasPattern: {
      SetAsCanvasPattern() = aOther.GetAsCanvasPattern();
      break;
    }
  }
}

nsString&
OwningStringOrFileOrDirectory::RawSetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eString;
  return mValue.mString.SetValue();
}

nsString&
OwningStringOrFileOrDirectory::SetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  Uninit();
  mType = eString;
  return mValue.mString.SetValue();
}

bool
OwningStringOrFileOrDirectory::TrySetToString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
  }
  return true;
}


void
OwningStringOrFileOrDirectory::DestroyString()
{
  MOZ_ASSERT(IsString(), "Wrong type!");
  mValue.mString.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::File>&
OwningStringOrFileOrDirectory::RawSetAsFile()
{
  if (mType == eFile) {
    return mValue.mFile.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eFile;
  return mValue.mFile.SetValue();
}

OwningNonNull<mozilla::dom::File>&
OwningStringOrFileOrDirectory::SetAsFile()
{
  if (mType == eFile) {
    return mValue.mFile.Value();
  }
  Uninit();
  mType = eFile;
  return mValue.mFile.SetValue();
}

bool
OwningStringOrFileOrDirectory::TrySetToFile(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::File>& memberSlot = RawSetAsFile();
    {
      nsresult rv = UnwrapObject<prototypes::id::File, mozilla::dom::File>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyFile();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningStringOrFileOrDirectory::DestroyFile()
{
  MOZ_ASSERT(IsFile(), "Wrong type!");
  mValue.mFile.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::Directory>&
OwningStringOrFileOrDirectory::RawSetAsDirectory()
{
  if (mType == eDirectory) {
    return mValue.mDirectory.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eDirectory;
  return mValue.mDirectory.SetValue();
}

OwningNonNull<mozilla::dom::Directory>&
OwningStringOrFileOrDirectory::SetAsDirectory()
{
  if (mType == eDirectory) {
    return mValue.mDirectory.Value();
  }
  Uninit();
  mType = eDirectory;
  return mValue.mDirectory.SetValue();
}

bool
OwningStringOrFileOrDirectory::TrySetToDirectory(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::Directory>& memberSlot = RawSetAsDirectory();
    {
      nsresult rv = UnwrapObject<prototypes::id::Directory, mozilla::dom::Directory>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyDirectory();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningStringOrFileOrDirectory::DestroyDirectory()
{
  MOZ_ASSERT(IsDirectory(), "Wrong type!");
  mValue.mDirectory.Destroy();
  mType = eUninitialized;
}




void
OwningStringOrFileOrDirectory::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eString: {
      DestroyString();
      break;
    }
    case eFile: {
      DestroyFile();
      break;
    }
    case eDirectory: {
      DestroyDirectory();
      break;
    }
  }
}

bool
OwningStringOrFileOrDirectory::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eFile: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mFile.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eDirectory: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mDirectory.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningStringOrFileOrDirectory::TraceUnion(JSTracer* trc)
{
}

void
OwningStringOrFileOrDirectory::operator=(const OwningStringOrFileOrDirectory& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eString: {
      SetAsString() = aOther.GetAsString();
      break;
    }
    case eFile: {
      SetAsFile() = aOther.GetAsFile();
      break;
    }
    case eDirectory: {
      SetAsDirectory() = aOther.GetAsDirectory();
      break;
    }
  }
}

nsString&
OwningStringOrInstallTriggerData::RawSetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eString;
  return mValue.mString.SetValue();
}

nsString&
OwningStringOrInstallTriggerData::SetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  Uninit();
  mType = eString;
  return mValue.mString.SetValue();
}

bool
OwningStringOrInstallTriggerData::TrySetToString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
  }
  return true;
}


void
OwningStringOrInstallTriggerData::DestroyString()
{
  MOZ_ASSERT(IsString(), "Wrong type!");
  mValue.mString.Destroy();
  mType = eUninitialized;
}




InstallTriggerData&
OwningStringOrInstallTriggerData::RawSetAsInstallTriggerData()
{
  if (mType == eInstallTriggerData) {
    return mValue.mInstallTriggerData.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eInstallTriggerData;
  return mValue.mInstallTriggerData.SetValue();
}

InstallTriggerData&
OwningStringOrInstallTriggerData::SetAsInstallTriggerData()
{
  if (mType == eInstallTriggerData) {
    return mValue.mInstallTriggerData.Value();
  }
  Uninit();
  mType = eInstallTriggerData;
  return mValue.mInstallTriggerData.SetValue();
}

bool
OwningStringOrInstallTriggerData::TrySetToInstallTriggerData(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    InstallTriggerData& memberSlot = RawSetAsInstallTriggerData();
    if (!IsConvertibleToDictionary(cx, value)) {
      DestroyInstallTriggerData();
      tryNext = true;
      return true;
    }

    if (!memberSlot.Init(cx, value, "Member of StringOrInstallTriggerData", passedToJSImpl)) {
      return false;
    }
  }
  return true;
}

void
OwningStringOrInstallTriggerData::DestroyInstallTriggerData()
{
  MOZ_ASSERT(IsInstallTriggerData(), "Wrong type!");
  mValue.mInstallTriggerData.Destroy();
  mType = eUninitialized;
}




void
OwningStringOrInstallTriggerData::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eString: {
      DestroyString();
      break;
    }
    case eInstallTriggerData: {
      DestroyInstallTriggerData();
      break;
    }
  }
}

bool
OwningStringOrInstallTriggerData::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eInstallTriggerData: {
      if (!mValue.mInstallTriggerData.Value().ToObjectInternal(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningStringOrInstallTriggerData::TraceUnion(JSTracer* trc)
{
}

void
OwningStringOrInstallTriggerData::operator=(const OwningStringOrInstallTriggerData& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eString: {
      SetAsString() = aOther.GetAsString();
      break;
    }
    case eInstallTriggerData: {
      SetAsInstallTriggerData() = aOther.GetAsInstallTriggerData();
      break;
    }
  }
}

nsString&
OwningStringOrUnsignedLong::RawSetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eString;
  return mValue.mString.SetValue();
}

nsString&
OwningStringOrUnsignedLong::SetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  Uninit();
  mType = eString;
  return mValue.mString.SetValue();
}

bool
OwningStringOrUnsignedLong::TrySetToString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
  }
  return true;
}


void
OwningStringOrUnsignedLong::DestroyString()
{
  MOZ_ASSERT(IsString(), "Wrong type!");
  mValue.mString.Destroy();
  mType = eUninitialized;
}




uint32_t&
OwningStringOrUnsignedLong::RawSetAsUnsignedLong()
{
  if (mType == eUnsignedLong) {
    return mValue.mUnsignedLong.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eUnsignedLong;
  return mValue.mUnsignedLong.SetValue();
}

uint32_t&
OwningStringOrUnsignedLong::SetAsUnsignedLong()
{
  if (mType == eUnsignedLong) {
    return mValue.mUnsignedLong.Value();
  }
  Uninit();
  mType = eUnsignedLong;
  return mValue.mUnsignedLong.SetValue();
}

bool
OwningStringOrUnsignedLong::TrySetToUnsignedLong(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    uint32_t& memberSlot = RawSetAsUnsignedLong();
    if (!ValueToPrimitive<uint32_t, eDefault>(cx, value, &memberSlot)) {
      return false;
    }
  }
  return true;
}

void
OwningStringOrUnsignedLong::DestroyUnsignedLong()
{
  MOZ_ASSERT(IsUnsignedLong(), "Wrong type!");
  mValue.mUnsignedLong.Destroy();
  mType = eUninitialized;
}




void
OwningStringOrUnsignedLong::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eString: {
      DestroyString();
      break;
    }
    case eUnsignedLong: {
      DestroyUnsignedLong();
      break;
    }
  }
}

bool
OwningStringOrUnsignedLong::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eUnsignedLong: {
      rval.setNumber(mValue.mUnsignedLong.Value());
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningStringOrUnsignedLong::TraceUnion(JSTracer* trc)
{
}

void
OwningStringOrUnsignedLong::operator=(const OwningStringOrUnsignedLong& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eString: {
      SetAsString() = aOther.GetAsString();
      break;
    }
    case eUnsignedLong: {
      SetAsUnsignedLong() = aOther.GetAsUnsignedLong();
      break;
    }
  }
}

OwningNonNull<mozilla::dom::TelephonyCall>&
OwningTelephonyCallOrTelephonyCallGroup::RawSetAsTelephonyCall()
{
  if (mType == eTelephonyCall) {
    return mValue.mTelephonyCall.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eTelephonyCall;
  return mValue.mTelephonyCall.SetValue();
}

OwningNonNull<mozilla::dom::TelephonyCall>&
OwningTelephonyCallOrTelephonyCallGroup::SetAsTelephonyCall()
{
  if (mType == eTelephonyCall) {
    return mValue.mTelephonyCall.Value();
  }
  Uninit();
  mType = eTelephonyCall;
  return mValue.mTelephonyCall.SetValue();
}

bool
OwningTelephonyCallOrTelephonyCallGroup::TrySetToTelephonyCall(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::TelephonyCall>& memberSlot = RawSetAsTelephonyCall();
    {
      nsresult rv = UnwrapObject<prototypes::id::TelephonyCall, mozilla::dom::TelephonyCall>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyTelephonyCall();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningTelephonyCallOrTelephonyCallGroup::DestroyTelephonyCall()
{
  MOZ_ASSERT(IsTelephonyCall(), "Wrong type!");
  mValue.mTelephonyCall.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::TelephonyCallGroup>&
OwningTelephonyCallOrTelephonyCallGroup::RawSetAsTelephonyCallGroup()
{
  if (mType == eTelephonyCallGroup) {
    return mValue.mTelephonyCallGroup.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eTelephonyCallGroup;
  return mValue.mTelephonyCallGroup.SetValue();
}

OwningNonNull<mozilla::dom::TelephonyCallGroup>&
OwningTelephonyCallOrTelephonyCallGroup::SetAsTelephonyCallGroup()
{
  if (mType == eTelephonyCallGroup) {
    return mValue.mTelephonyCallGroup.Value();
  }
  Uninit();
  mType = eTelephonyCallGroup;
  return mValue.mTelephonyCallGroup.SetValue();
}

bool
OwningTelephonyCallOrTelephonyCallGroup::TrySetToTelephonyCallGroup(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::TelephonyCallGroup>& memberSlot = RawSetAsTelephonyCallGroup();
    {
      nsresult rv = UnwrapObject<prototypes::id::TelephonyCallGroup, mozilla::dom::TelephonyCallGroup>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyTelephonyCallGroup();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningTelephonyCallOrTelephonyCallGroup::DestroyTelephonyCallGroup()
{
  MOZ_ASSERT(IsTelephonyCallGroup(), "Wrong type!");
  mValue.mTelephonyCallGroup.Destroy();
  mType = eUninitialized;
}




void
OwningTelephonyCallOrTelephonyCallGroup::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eTelephonyCall: {
      DestroyTelephonyCall();
      break;
    }
    case eTelephonyCallGroup: {
      DestroyTelephonyCallGroup();
      break;
    }
  }
}

bool
OwningTelephonyCallOrTelephonyCallGroup::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eTelephonyCall: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mTelephonyCall.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eTelephonyCallGroup: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mTelephonyCallGroup.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningTelephonyCallOrTelephonyCallGroup::TraceUnion(JSTracer* trc)
{
}

void
OwningTelephonyCallOrTelephonyCallGroup::operator=(const OwningTelephonyCallOrTelephonyCallGroup& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eTelephonyCall: {
      SetAsTelephonyCall() = aOther.GetAsTelephonyCall();
      break;
    }
    case eTelephonyCallGroup: {
      SetAsTelephonyCallGroup() = aOther.GetAsTelephonyCallGroup();
      break;
    }
  }
}

OwningNonNull<mozilla::dom::Text>&
OwningTextOrElementOrDocument::RawSetAsText()
{
  if (mType == eText) {
    return mValue.mText.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eText;
  return mValue.mText.SetValue();
}

OwningNonNull<mozilla::dom::Text>&
OwningTextOrElementOrDocument::SetAsText()
{
  if (mType == eText) {
    return mValue.mText.Value();
  }
  Uninit();
  mType = eText;
  return mValue.mText.SetValue();
}

bool
OwningTextOrElementOrDocument::TrySetToText(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::Text>& memberSlot = RawSetAsText();
    {
      nsresult rv = UnwrapObject<prototypes::id::Text, mozilla::dom::Text>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyText();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningTextOrElementOrDocument::DestroyText()
{
  MOZ_ASSERT(IsText(), "Wrong type!");
  mValue.mText.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::Element>&
OwningTextOrElementOrDocument::RawSetAsElement()
{
  if (mType == eElement) {
    return mValue.mElement.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eElement;
  return mValue.mElement.SetValue();
}

OwningNonNull<mozilla::dom::Element>&
OwningTextOrElementOrDocument::SetAsElement()
{
  if (mType == eElement) {
    return mValue.mElement.Value();
  }
  Uninit();
  mType = eElement;
  return mValue.mElement.SetValue();
}

bool
OwningTextOrElementOrDocument::TrySetToElement(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::Element>& memberSlot = RawSetAsElement();
    {
      nsresult rv = UnwrapObject<prototypes::id::Element, mozilla::dom::Element>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyElement();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningTextOrElementOrDocument::DestroyElement()
{
  MOZ_ASSERT(IsElement(), "Wrong type!");
  mValue.mElement.Destroy();
  mType = eUninitialized;
}




OwningNonNull<nsIDocument>&
OwningTextOrElementOrDocument::RawSetAsDocument()
{
  if (mType == eDocument) {
    return mValue.mDocument.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eDocument;
  return mValue.mDocument.SetValue();
}

OwningNonNull<nsIDocument>&
OwningTextOrElementOrDocument::SetAsDocument()
{
  if (mType == eDocument) {
    return mValue.mDocument.Value();
  }
  Uninit();
  mType = eDocument;
  return mValue.mDocument.SetValue();
}

bool
OwningTextOrElementOrDocument::TrySetToDocument(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<nsIDocument>& memberSlot = RawSetAsDocument();
    {
      nsresult rv = UnwrapObject<prototypes::id::Document, nsIDocument>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyDocument();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningTextOrElementOrDocument::DestroyDocument()
{
  MOZ_ASSERT(IsDocument(), "Wrong type!");
  mValue.mDocument.Destroy();
  mType = eUninitialized;
}




void
OwningTextOrElementOrDocument::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eText: {
      DestroyText();
      break;
    }
    case eElement: {
      DestroyElement();
      break;
    }
    case eDocument: {
      DestroyDocument();
      break;
    }
  }
}

bool
OwningTextOrElementOrDocument::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eText: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mText.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eElement: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mElement.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eDocument: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mDocument.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningTextOrElementOrDocument::TraceUnion(JSTracer* trc)
{
}

void
OwningTextOrElementOrDocument::operator=(const OwningTextOrElementOrDocument& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eText: {
      SetAsText() = aOther.GetAsText();
      break;
    }
    case eElement: {
      SetAsElement() = aOther.GetAsElement();
      break;
    }
    case eDocument: {
      SetAsDocument() = aOther.GetAsDocument();
      break;
    }
  }
}

uint32_t&
OwningUnsignedLongOrString::RawSetAsUnsignedLong()
{
  if (mType == eUnsignedLong) {
    return mValue.mUnsignedLong.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eUnsignedLong;
  return mValue.mUnsignedLong.SetValue();
}

uint32_t&
OwningUnsignedLongOrString::SetAsUnsignedLong()
{
  if (mType == eUnsignedLong) {
    return mValue.mUnsignedLong.Value();
  }
  Uninit();
  mType = eUnsignedLong;
  return mValue.mUnsignedLong.SetValue();
}

bool
OwningUnsignedLongOrString::TrySetToUnsignedLong(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    uint32_t& memberSlot = RawSetAsUnsignedLong();
    if (!ValueToPrimitive<uint32_t, eDefault>(cx, value, &memberSlot)) {
      return false;
    }
  }
  return true;
}

void
OwningUnsignedLongOrString::DestroyUnsignedLong()
{
  MOZ_ASSERT(IsUnsignedLong(), "Wrong type!");
  mValue.mUnsignedLong.Destroy();
  mType = eUninitialized;
}




nsString&
OwningUnsignedLongOrString::RawSetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eString;
  return mValue.mString.SetValue();
}

nsString&
OwningUnsignedLongOrString::SetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  Uninit();
  mType = eString;
  return mValue.mString.SetValue();
}

bool
OwningUnsignedLongOrString::TrySetToString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
  }
  return true;
}


void
OwningUnsignedLongOrString::DestroyString()
{
  MOZ_ASSERT(IsString(), "Wrong type!");
  mValue.mString.Destroy();
  mType = eUninitialized;
}




void
OwningUnsignedLongOrString::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eUnsignedLong: {
      DestroyUnsignedLong();
      break;
    }
    case eString: {
      DestroyString();
      break;
    }
  }
}

bool
OwningUnsignedLongOrString::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eUnsignedLong: {
      rval.setNumber(mValue.mUnsignedLong.Value());
      return true;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningUnsignedLongOrString::TraceUnion(JSTracer* trc)
{
}

void
OwningUnsignedLongOrString::operator=(const OwningUnsignedLongOrString& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eUnsignedLong: {
      SetAsUnsignedLong() = aOther.GetAsUnsignedLong();
      break;
    }
    case eString: {
      SetAsString() = aOther.GetAsString();
      break;
    }
  }
}

uint16_t&
OwningUnsignedShortOrObject::RawSetAsUnsignedShort()
{
  if (mType == eUnsignedShort) {
    return mValue.mUnsignedShort.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eUnsignedShort;
  return mValue.mUnsignedShort.SetValue();
}

uint16_t&
OwningUnsignedShortOrObject::SetAsUnsignedShort()
{
  if (mType == eUnsignedShort) {
    return mValue.mUnsignedShort.Value();
  }
  Uninit();
  mType = eUnsignedShort;
  return mValue.mUnsignedShort.SetValue();
}

bool
OwningUnsignedShortOrObject::TrySetToUnsignedShort(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    uint16_t& memberSlot = RawSetAsUnsignedShort();
    if (!ValueToPrimitive<uint16_t, eDefault>(cx, value, &memberSlot)) {
      return false;
    }
  }
  return true;
}

void
OwningUnsignedShortOrObject::DestroyUnsignedShort()
{
  MOZ_ASSERT(IsUnsignedShort(), "Wrong type!");
  mValue.mUnsignedShort.Destroy();
  mType = eUninitialized;
}




JSObject*&
OwningUnsignedShortOrObject::RawSetAsObject()
{
  if (mType == eObject) {
    return mValue.mObject.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eObject;
  return mValue.mObject.SetValue();
}

JSObject*&
OwningUnsignedShortOrObject::SetAsObject()
{
  if (mType == eObject) {
    return mValue.mObject.Value();
  }
  Uninit();
  mType = eObject;
  return mValue.mObject.SetValue();
}


void
OwningUnsignedShortOrObject::DestroyObject()
{
  MOZ_ASSERT(IsObject(), "Wrong type!");
  mValue.mObject.Destroy();
  mType = eUninitialized;
}




void
OwningUnsignedShortOrObject::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eUnsignedShort: {
      DestroyUnsignedShort();
      break;
    }
    case eObject: {
      DestroyObject();
      break;
    }
  }
}

bool
OwningUnsignedShortOrObject::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eUnsignedShort: {
      rval.setInt32(int32_t(mValue.mUnsignedShort.Value()));
      return true;
      break;
    }
    case eObject: {
      JS::ExposeObjectToActiveJS(mValue.mObject.Value());
      rval.setObject(*mValue.mObject.Value());
      if (!MaybeWrapObjectValue(cx, rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningUnsignedShortOrObject::TraceUnion(JSTracer* trc)
{
  switch (mType) {
    case eObject: {
      JS_CallUnbarrieredObjectTracer(trc, &mValue.mObject.Value(), "mValue.mObject");
      break;
    }
    default: {
      break;
    }
  }
}

OwningNonNull<mozilla::dom::VideoTrack>&
OwningVideoTrackOrAudioTrackOrTextTrack::RawSetAsVideoTrack()
{
  if (mType == eVideoTrack) {
    return mValue.mVideoTrack.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eVideoTrack;
  return mValue.mVideoTrack.SetValue();
}

OwningNonNull<mozilla::dom::VideoTrack>&
OwningVideoTrackOrAudioTrackOrTextTrack::SetAsVideoTrack()
{
  if (mType == eVideoTrack) {
    return mValue.mVideoTrack.Value();
  }
  Uninit();
  mType = eVideoTrack;
  return mValue.mVideoTrack.SetValue();
}

bool
OwningVideoTrackOrAudioTrackOrTextTrack::TrySetToVideoTrack(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::VideoTrack>& memberSlot = RawSetAsVideoTrack();
    {
      nsresult rv = UnwrapObject<prototypes::id::VideoTrack, mozilla::dom::VideoTrack>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyVideoTrack();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningVideoTrackOrAudioTrackOrTextTrack::DestroyVideoTrack()
{
  MOZ_ASSERT(IsVideoTrack(), "Wrong type!");
  mValue.mVideoTrack.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::AudioTrack>&
OwningVideoTrackOrAudioTrackOrTextTrack::RawSetAsAudioTrack()
{
  if (mType == eAudioTrack) {
    return mValue.mAudioTrack.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eAudioTrack;
  return mValue.mAudioTrack.SetValue();
}

OwningNonNull<mozilla::dom::AudioTrack>&
OwningVideoTrackOrAudioTrackOrTextTrack::SetAsAudioTrack()
{
  if (mType == eAudioTrack) {
    return mValue.mAudioTrack.Value();
  }
  Uninit();
  mType = eAudioTrack;
  return mValue.mAudioTrack.SetValue();
}

bool
OwningVideoTrackOrAudioTrackOrTextTrack::TrySetToAudioTrack(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::AudioTrack>& memberSlot = RawSetAsAudioTrack();
    {
      nsresult rv = UnwrapObject<prototypes::id::AudioTrack, mozilla::dom::AudioTrack>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyAudioTrack();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningVideoTrackOrAudioTrackOrTextTrack::DestroyAudioTrack()
{
  MOZ_ASSERT(IsAudioTrack(), "Wrong type!");
  mValue.mAudioTrack.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::TextTrack>&
OwningVideoTrackOrAudioTrackOrTextTrack::RawSetAsTextTrack()
{
  if (mType == eTextTrack) {
    return mValue.mTextTrack.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eTextTrack;
  return mValue.mTextTrack.SetValue();
}

OwningNonNull<mozilla::dom::TextTrack>&
OwningVideoTrackOrAudioTrackOrTextTrack::SetAsTextTrack()
{
  if (mType == eTextTrack) {
    return mValue.mTextTrack.Value();
  }
  Uninit();
  mType = eTextTrack;
  return mValue.mTextTrack.SetValue();
}

bool
OwningVideoTrackOrAudioTrackOrTextTrack::TrySetToTextTrack(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::TextTrack>& memberSlot = RawSetAsTextTrack();
    {
      nsresult rv = UnwrapObject<prototypes::id::TextTrack, mozilla::dom::TextTrack>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyTextTrack();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningVideoTrackOrAudioTrackOrTextTrack::DestroyTextTrack()
{
  MOZ_ASSERT(IsTextTrack(), "Wrong type!");
  mValue.mTextTrack.Destroy();
  mType = eUninitialized;
}




void
OwningVideoTrackOrAudioTrackOrTextTrack::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eVideoTrack: {
      DestroyVideoTrack();
      break;
    }
    case eAudioTrack: {
      DestroyAudioTrack();
      break;
    }
    case eTextTrack: {
      DestroyTextTrack();
      break;
    }
  }
}

bool
OwningVideoTrackOrAudioTrackOrTextTrack::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eVideoTrack: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mVideoTrack.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eAudioTrack: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mAudioTrack.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eTextTrack: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mTextTrack.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningVideoTrackOrAudioTrackOrTextTrack::TraceUnion(JSTracer* trc)
{
}

void
OwningVideoTrackOrAudioTrackOrTextTrack::operator=(const OwningVideoTrackOrAudioTrackOrTextTrack& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eVideoTrack: {
      SetAsVideoTrack() = aOther.GetAsVideoTrack();
      break;
    }
    case eAudioTrack: {
      SetAsAudioTrack() = aOther.GetAsAudioTrack();
      break;
    }
    case eTextTrack: {
      SetAsTextTrack() = aOther.GetAsTextTrack();
      break;
    }
  }
}

nsRefPtr<nsIDOMWindow>&
OwningWindowProxyOrMessagePort::RawSetAsWindowProxy()
{
  if (mType == eWindowProxy) {
    return mValue.mWindowProxy.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eWindowProxy;
  return mValue.mWindowProxy.SetValue();
}

nsRefPtr<nsIDOMWindow>&
OwningWindowProxyOrMessagePort::SetAsWindowProxy()
{
  if (mType == eWindowProxy) {
    return mValue.mWindowProxy.Value();
  }
  Uninit();
  mType = eWindowProxy;
  return mValue.mWindowProxy.SetValue();
}

bool
OwningWindowProxyOrMessagePort::TrySetToWindowProxy(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsRefPtr<nsIDOMWindow>& memberSlot = RawSetAsWindowProxy();
    nsRefPtr<nsIDOMWindow> mWindowProxyHolder;
    JS::Rooted<JS::Value> tmpVal(cx, value);
    nsIDOMWindow* tmp;
    if (NS_FAILED(UnwrapArg<nsIDOMWindow>(cx, value, &tmp, static_cast<nsIDOMWindow**>(getter_AddRefs(mWindowProxyHolder)), &tmpVal))) {
      DestroyWindowProxy();
      tryNext = true;
      return true;
    }
    MOZ_ASSERT(tmp);
    memberSlot = tmp;
  }
  return true;
}

void
OwningWindowProxyOrMessagePort::DestroyWindowProxy()
{
  MOZ_ASSERT(IsWindowProxy(), "Wrong type!");
  mValue.mWindowProxy.Destroy();
  mType = eUninitialized;
}




OwningNonNull<mozilla::dom::MessagePortBase>&
OwningWindowProxyOrMessagePort::RawSetAsMessagePort()
{
  if (mType == eMessagePort) {
    return mValue.mMessagePort.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eMessagePort;
  return mValue.mMessagePort.SetValue();
}

OwningNonNull<mozilla::dom::MessagePortBase>&
OwningWindowProxyOrMessagePort::SetAsMessagePort()
{
  if (mType == eMessagePort) {
    return mValue.mMessagePort.Value();
  }
  Uninit();
  mType = eMessagePort;
  return mValue.mMessagePort.SetValue();
}

bool
OwningWindowProxyOrMessagePort::TrySetToMessagePort(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::MessagePortBase>& memberSlot = RawSetAsMessagePort();
    {
      nsresult rv = UnwrapObject<prototypes::id::MessagePort, mozilla::dom::MessagePortBase>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroyMessagePort();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningWindowProxyOrMessagePort::DestroyMessagePort()
{
  MOZ_ASSERT(IsMessagePort(), "Wrong type!");
  mValue.mMessagePort.Destroy();
  mType = eUninitialized;
}




void
OwningWindowProxyOrMessagePort::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case eWindowProxy: {
      DestroyWindowProxy();
      break;
    }
    case eMessagePort: {
      DestroyMessagePort();
      break;
    }
  }
}

bool
OwningWindowProxyOrMessagePort::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case eWindowProxy: {
      if (!WrapObject(cx, mValue.mWindowProxy.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    case eMessagePort: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mMessagePort.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningWindowProxyOrMessagePort::TraceUnion(JSTracer* trc)
{
}

void
OwningWindowProxyOrMessagePort::operator=(const OwningWindowProxyOrMessagePort& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case eWindowProxy: {
      SetAsWindowProxy() = aOther.GetAsWindowProxy();
      break;
    }
    case eMessagePort: {
      SetAsMessagePort() = aOther.GetAsMessagePort();
      break;
    }
  }
}

OwningNonNull<mozilla::dom::mozContact>&
OwningmozContactOrString::RawSetAsmozContact()
{
  if (mType == emozContact) {
    return mValue.mmozContact.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = emozContact;
  return mValue.mmozContact.SetValue();
}

OwningNonNull<mozilla::dom::mozContact>&
OwningmozContactOrString::SetAsmozContact()
{
  if (mType == emozContact) {
    return mValue.mmozContact.Value();
  }
  Uninit();
  mType = emozContact;
  return mValue.mmozContact.SetValue();
}

bool
OwningmozContactOrString::TrySetTomozContact(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    OwningNonNull<mozilla::dom::mozContact>& memberSlot = RawSetAsmozContact();
    {
      nsresult rv = UnwrapObject<prototypes::id::mozContact, mozilla::dom::mozContact>(&value.toObject(), memberSlot);
      if (NS_FAILED(rv)) {
        DestroymozContact();
        tryNext = true;
        return true;
      }
    }
  }
  return true;
}

void
OwningmozContactOrString::DestroymozContact()
{
  MOZ_ASSERT(IsmozContact(), "Wrong type!");
  mValue.mmozContact.Destroy();
  mType = eUninitialized;
}




nsString&
OwningmozContactOrString::RawSetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  MOZ_ASSERT(mType == eUninitialized);
  mType = eString;
  return mValue.mString.SetValue();
}

nsString&
OwningmozContactOrString::SetAsString()
{
  if (mType == eString) {
    return mValue.mString.Value();
  }
  Uninit();
  mType = eString;
  return mValue.mString.SetValue();
}

bool
OwningmozContactOrString::TrySetToString(JSContext* cx, JS::Handle<JS::Value> value, bool& tryNext, bool passedToJSImpl)
{
  tryNext = false;
  { // scope for memberSlot
    nsString& memberSlot = RawSetAsString();
    if (!ConvertJSValueToString(cx, value, eStringify, eStringify, memberSlot)) {
      return false;
    }
  }
  return true;
}


void
OwningmozContactOrString::DestroyString()
{
  MOZ_ASSERT(IsString(), "Wrong type!");
  mValue.mString.Destroy();
  mType = eUninitialized;
}




void
OwningmozContactOrString::Uninit()
{
  switch (mType) {
    case eUninitialized: {
      break;
    }
    case emozContact: {
      DestroymozContact();
      break;
    }
    case eString: {
      DestroyString();
      break;
    }
  }
}

bool
OwningmozContactOrString::ToJSVal(JSContext* cx, JS::Handle<JSObject*> scopeObj, JS::MutableHandle<JS::Value> rval) const
{
  switch (mType) {
    case eUninitialized: {
      return false;
      break;
    }
    case emozContact: {
      if (!WrapNewBindingObject(cx, scopeObj, mValue.mmozContact.Value(), rval)) {
        MOZ_ASSERT(true || JS_IsExceptionPending(cx));
        return false;
      }
      return true;
      break;
    }
    case eString: {
      if (!xpc::NonVoidStringToJsval(cx, mValue.mString.Value(), rval)) {
        return false;
      }
      return true;
      break;
    }
    default: {
      return false;
      break;
    }
  }

  return false;
}

void
OwningmozContactOrString::TraceUnion(JSTracer* trc)
{
}

void
OwningmozContactOrString::operator=(const OwningmozContactOrString& aOther)
{
  switch (aOther.mType) {
    case eUninitialized: {
      MOZ_ASSERT(mType == eUninitialized,
                 "We need to destroy ourselves?");
      break;
    }
    case emozContact: {
      SetAsmozContact() = aOther.GetAsmozContact();
      break;
    }
    case eString: {
      SetAsString() = aOther.GetAsString();
      break;
    }
  }
}
} // namespace dom
} // namespace mozilla

